create or replace PROCEDURE PR_ALTA_CENTRALDIST (

p_idCDist IN CENTRALDIST.idCDist%TYPE,
p_adreca IN CENTRALDIST.adreca%TYPE,
p_codiPostal IN CENTRALDIST.codiPostal%TYPE,
p_localitat IN CENTRALDIST.localitat%TYPE,
p_pais IN CENTRALDIST.pais%TYPE,
p_maxKWh CENTRALDIST.maxKWh%TYPE,
p_dataFuncionament CENTRALDIST.dataFuncionament%TYPE,

RSP OUT NOCOPY TAULALOG.RSP%TYPE


)

--/*****************************************************************************
--  PROCEDIMENT:  PR_ALTA_CENTRALDIST
--
--  Dona d'alta una central de distribuci.
--
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la
--  TAULALOG el resultat de la operaci.
--
--  ENTRADA : p_IDCDIST NUMBER(10), p_ADRECA VARCHAR(30), p_CODIPOSTAL CHAR(5),  
--          p_LOCALITAT VARCHAR2(30), p_PAIS VARCHAR2(20), p_MAXKWH NUMBER(12,1)  
--          p_DATAFUNCIONAMENT   DATE .
--
--  SORTIDA : RSP VARCHAR2(300) indicant si ha anat correcte o hi ha errors.

--  Si el  parmetre idCdist = 0, llavors s'agafa el prxim valor de seqncia.
--  Els camps adreca,codipostal, localitat, pais no poden repetir-se. (UNIQUE)
--
--  DataBaixa al donar d'alta ser null.
--
--    RSP = OK : S'inserta registre.
--
--    RSP = ERROR + motiu, si hi ha algun error.
--
--    Motius errors tractats :
--      - ERROR: HI HA NULS NO PERMESOS ALS PARAMETRES D'ENTRADA
--      - ERROR: CLAU IDCDIST idCdist DUPLICADA A LA TAULA CENTRALDIST.
--  	- ERROR: DATA DE FUNCIONAMENT SUPERIOR A LA ACTUAL
--	- ERROR: PARAMETRE MAX DE KWH AMB VALOR NEGATIU
--      - ERROR: JA HI HA UNA CENTRAL DE DISTRIBUCIO AMB AQUESTA ADRECA
--      - others
--
--******************************************************************************/


IS
        -- variable dataBaixa de la c. distribuci que s'insertar com a null
        dataBaixaCD CENTRALDIST.dataBaixa%TYPE := null;

	-- variables errors
        e_camps_nuls EXCEPTION;
        e_idCDist_jaexisteix EXCEPTION;
        e_dataFuncionament_erronia EXCEPTION;
        e_maxKWh_erroni EXCEPTION;
        e_adreca_repetida EXCEPTION;

        -- variables insert taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_ALTA_CENTRALDIST';
	paramEntrada taulaLog.paramEntrada%TYPE := 'idCDist: ' || p_idCDist || ', adreca: ' || p_adreca || ', codiPostal: '
            || p_codiPostal || ', localitat: ' || p_localitat  ||' ,pais: '|| p_pais|| ', maxKWh : ' || p_maxKWh
            ||' , dataFuncionament: '|| p_dataFuncionament;
      	paramSortida taulaLog.paramSortida%TYPE := 'RSP';

        -- variable d's divers (comptadors, etc)
        numElements number :=0;
        numIdSeq number;

BEGIN
      RSP  := 'OK';

      -- VERIFIQUEM ELS NULS DELS PARAMETRES D'ENTRADA
      IF  ((p_idCDist is null) or (p_adreca is null) or (p_codiPostal is null) or ( p_localitat is null) or (p_pais is null)
          or (p_maxKWh is null) or (p_dataFuncionament is null))
          THEN RAISE e_camps_nuls;
      END IF;

      -- VERIFIQUEM QUE NO EXISTEIXI IDCDIST
      SELECT COUNT(*) INTO numElements FROM CENTRALDIST
        WHERE p_idCDist = CENTRALDIST.idCDist;
      IF numElements <> 0
        THEN RAISE e_idCDist_jaexisteix;
      END IF;

      -- VERIFIQUEM QUE DATA FUNCIONAMENT SIGUI INFERIOR A LA ACTUAL
      IF p_dataFuncionament > sysdate
        THEN RAISE e_dataFuncionament_erronia;
      END IF;


     -- VERIFIQUEM QUE MAXKWH ES UNA QUANTITAT POSITIVA
     IF p_maxKWh < 0
      THEN RAISE e_maxKWh_erroni;
     END IF;

      -- VERIFIQUEM QUE NO ES REPETEIX LA CLAU UNICA DE ADREA
      SELECT COUNT(*) INTO numElements FROM CENTRALDIST
       WHERE p_adreca = adreca and p_codiPostal = codiPostal and p_localitat = localitat and p_pais = pais;
      IF numElements <> 0
       THEN RAISE e_adreca_repetida;
       END IF;

       -- SI NO HI HA ERRORS, INSERTEM, i si p_idCDist s zero, 
       -- anotem al lot el num de la seqncia per saber amb quina idCDist s'ha insertat
       IF p_idCDist <> 0 THEN
	INSERT INTO CENTRALDIST VALUES (p_idCDist, p_adreca, p_codiPostal, p_localitat,p_pais, p_maxKWh, p_dataFuncionament, dataBaixaCD);
	ELSE
	 INSERT INTO CENTRALDIST VALUES (p_idCDist, p_adreca, p_codiPostal, p_localitat,p_pais, p_maxKWh, p_dataFuncionament, dataBaixaCD)
			RETURNING idCDist INTO NUMIDSEQ;
		ParamEntrada := '(numIdSeq :'||numIdSeq||') Params :'||paramEntrada;
	END IF;
	
	RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);

        COMMIT;
        if p_idCdist <> 0 THEN
        DBMS_OUTPUT.PUT_LINE('C. DISTRIBUCIO AMB ID '||P_IDCDIST||' AFEGIDA A LA TAULA CENTRALDIST');
	ELSE
	DBMS_OUTPUT.PUT_LINE('C. DISTRIBUCIO AMB ID '||numidSEq||' AFEGIDA A LA TAULA CENTRALDIST');
	END IF;


EXCEPTION

	WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS NO PERMESOS ALS PARAMETRES D''ENTRADA';
               PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
             	Rollback;

	WHEN e_idCDist_jaexisteix THEN
        	RSP := 'ERROR: IDCDIST ' || p_idcDist|| ' JA EXISTEIX A LA TAULA CENTRALDIST';
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
             	Rollback;

      WHEN e_dataFuncionament_erronia THEN
        	RSP := 'ERROR: DATA DE FUNCIONAMENT ' || p_dataFuncionament || ' SUPERIOR A LA ACTUAL '|| SYSDATE;
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
	        Rollback;

       WHEN e_maxKWh_erroni THEN
        	RSP := 'ERROR: PARAMETRE MAX DE KWH AMB VALOR NEGATIU';
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
	        Rollback;

        WHEN e_adreca_repetida THEN
        	RSP := 'ERROR: JA HI HA UNA CENTRAL DE DISTRIBUCIO AMB AQUESTA ADRECA';
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
	        Rollback;

	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
        	ROLLBACK;

       	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR. PROCEDIMENT P_ALTA_COMPTADOR - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
        	ROLLBACK;

END;
/