create or replace PROCEDURE  PR_ALTA_CENTRALPROD (

p_idCProd IN CENTRALPROD.idCProd%TYPE ,
p_adreca IN CENTRALPROD.adreca%TYPE,
p_codiPostal IN CENTRALPROD.codiPostal%TYPE,
p_localitat IN CENTRALPROD.localitat%TYPE,
p_pais IN CENTRALPROD.pais%TYPE,
p_maxKWh IN CENTRALPROD.maxKWh%TYPE,
p_dataDarreraInspecc IN CENTRALPROD.dataDarreraInspecc%TYPE,
p_dataFuncionament IN CENTRALPROD.dataFuncionament%TYPE,
p_tipusCProd IN CENTRALPROD.tipusCProd%TYPE,
p_numAux IN CPNUCLEAR.energiaMin%TYPE,

RSP OUT NOCOPY TAULALOG.RSP%TYPE


)

--/*****************************************************************************
--  PROCEDIMENT:  PR_ALTA_CENTRALPROD
--
--  Dona d'alta una central de producci i en funci del tipus que sigui,
--  en aquest mateix procediment s'inserta a una de les taules( CPNUCLEAR,
--  CPEOLICA, CPSOLAR, CPTERMICA o CPCARBO) vinculant-les amb idCProd.
--
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la
--  TAULALOG el resultat de la operaci.
--
--  ENTRADA : P_IDCPROD NUMBER(10),  p_ ADRECA VARCHAR(30), p_CODIPOSTAL CHAR(5) 
--       p_LOCALITAT VARCHAR2(30), p_PAIS VARCHAR2(20), p_MAXKWH  NUMBER(12,1),  
--       p_DATADARRERAINSPECC DATE, p_DATAFUNCIONAMENT DATE, 
--       p_TIPUSCPROD VARCHAR(10), p_NUMAUX 
--
-- SORTIDA : RSP indicant si ha anat correcte o si ha hagut errors.
--
--  NOTA: si idCProd = 0, s'agafa de la seqncia S_IDCPROD
--  TipusCProd ha de ser : 'NUCLEAR', 'EOLICA', ,SOLAR', 'TERMICA' O 'CARBO'
--  NumAux ser els valors de :
--          * Numero de molins, en cas que tipusCProd sigui CPEolica
--          * Numero de panels en cas que tipusCProd sigui CPSolar
--          * Energia minima pel funcionament si es CPNUCLEAR.
--          * En altre cas, no s'utilitzar el valor.
--
--    Data baixa es posar com a nul al ser procediment d'alta.
--
--    RSP = OK : S'inserta registre.
--
--    RSP = ERROR + motiu, si hi ha algn error.
--
--    Motius errors tractats:
--      - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA.
--      - ERROR: CLAU IDCPROD idCProduccio DUPLICADA A LA TAULA CENTRALPROD.
--      - ERROR: IDCPROD HA DE SER POSITIU.
--  	- ERROR: DATA DE FUNCIONAMENT  p_dataFuncionament  SUPERIOR A LA ACTUAL sysdate
--      - ERROR: DATA  DARRERA INSPECCIO p_dataDarreraInspecc SUPERIOR A LA ACTUAL sysdate
--      - ERROR: PARAMETRE MAX DE KWH AMB VALOR NEGATIU.
--      - ERROR: TIPUS  DIFERENT DE 'EOLICA', 'NUCLEAR','CARBO','SOLAR' I 'TERMICA'
--	- ERROR: PARAMETRE NUMAUX AMB VALOR NEGATIU.
--      - Altres
--
--******************************************************************************/


IS

        -- variables de camps a insertar a taula
        idCProduccio CENTRALPROD.idCProd%TYPE;
        -- databaixa s'insertar com a nul
        dataBaixaCP CENTRALPROD.dataBaixa%TYPE := null;

	-- variables errors
        e_camps_nuls EXCEPTION;
        e_idCProd_negatiu EXCEPTION;
        e_dataFuncionament_erronia EXCEPTION;
        e_dataDarreraInspecc_erronia EXCEPTION;
        e_maxKWh_erroni EXCEPTION;
        e_tipusCProd_erroni EXCEPTION;
        e_numAux_erroni EXCEPTION;

        -- variables insert taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_ALTA_CENTRALPROD';
	paramEntrada taulaLog.paramEntrada%TYPE := 'idCProd: ' || p_idCProd || ', adreca: ' || p_adreca || ', codiPostal: '
            || p_codiPostal || ', localitat: ' || p_localitat  ||' ,pais: '|| p_pais|| ', maxKWh : ' || p_maxKWh
            ||' dataDarreraInspecc:' ||p_dataDarreraInspecc||' , dataFuncionament: '|| p_dataFuncionament
            || ' ,tipusCProd :' ||p_tipusCProd ||', numAux :' || p_numAux;

	paramSortida taulaLog.paramSortida%TYPE := 'RSP';

        -- variable comptador i d's divers.
        numElements number :=0;
       

BEGIN

    RSP := 'OK';

    -- VERIFIQUEM QUE IDCPROD REBUT S POSITIU
    IF p_idCProd < 0 THEN
       RAISE e_idCProd_negatiu;
    END IF;

    -- SI IDCPROD S ZERO AGAFEM EL SEGENT VALOR DE LA SEQNCIA
    --  EN CAS CONTRARI, AGAFEM EL VALOR PASSAT PER PARMETRE
    IF p_idCProd = 0
    THEN
       SELECT S_IDCPROD.NEXTVAL INTO idCProduccio
	FROM DUAL  ;
        ParamEntrada := '(numSeqncia :'||idCproduccio||') Params :'||paramEntrada;
    ELSE idCProduccio := p_idCProd;
    END IF;

    -- VERIFIQUEM ELS NULS DELS PARAMETRES D'ENTRADA
    IF  ((p_adreca is null) or (p_codiPostal is null) or ( p_localitat is null) or (p_pais is null)
          or (p_maxKWh is null) or (p_dataDarreraInspecc is null) or (p_dataFuncionament is null)
          or (p_tipusCProd is null) or (p_numAux is null))
    THEN RAISE e_camps_nuls;
    END IF;

    -- VERIFIQUEM QUE DATA FUNCIONAMENT SIGUI INFERIOR O IGUAL A LA ACTUAL
    IF p_dataFuncionament > sysdate
    THEN RAISE e_dataFuncionament_erronia;
    END IF;

     -- VERIFIQUEM QUE DATA DE DARRERA INSPECCIO SIGUI INFERIOR O IGUAL A LA ACTUAL
    IF p_dataDarreraInspecc > sysdate
    THEN RAISE e_dataDarreraInspecc_erronia;
    END IF;

     -- VERIFIQUEM QUE MAXKWH ES UNA QUANTITAT POSITIVA
     IF p_maxKWh < 0
      THEN RAISE e_maxKWh_erroni;
     END IF;

      -- VERIFIQUEM QUE EL TIPUS DE CENTRAL ES CORRECTE
      IF p_tipusCProd <> 'NUCLEAR' AND p_tipusCProd <> 'EOLICA' AND p_tipusCProd <> 'SOLAR'
          AND p_tipusCProd <> 'TERMICA' AND p_tipusCProd <> 'CARBO'
      THEN RAISE e_tipusCProd_erroni;
      END IF;

      -- VERIFIQUEM QUE EL PARAMETRE NUMAUX ES POSITIU
      IF p_numAux < 0
      THEN RAISE e_numAux_erroni;
      END IF;

      -- SI NO HI HA ERRORS, INSERTEM 
      INSERT INTO CENTRALPROD VALUES (idCProduccio, p_adreca, p_codiPostal, p_localitat,p_pais, p_maxKWh,
        p_dataDarreraInspecc, p_dataFuncionament, p_tipusCProd, dataBaixaCP);

      PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
      DBMS_OUTPUT.PUT_LINE('OK. REGISTRE AFEGIT A LA TAULA: CENTRALPROD. IDCPROD ('|| idCProduccio|| ') TIPUS :'|| p_tipusCProd);

      -- COM ES ALTA DE CENTRAL, SEGONS EL TIPUS, DONEM D'ALTA TAMB LA TAULA DEL TIPUS DE CENTRAL
      -- AMB VALORS  IDCPROD, I NUMAUX REBUTS PER PARMETRE.
      -- ELS CAMPS AMB KGEMISSIONS, O KG REBUIG SERAN ZERO.

        IF p_tipusCProd = 'NUCLEAR'
          THEN PR_ALTA_CPNUCLEAR(idCProduccio,p_numAux,0,RSP);
        END IF;

        IF p_tipusCProd = 'EOLICA'
         THEN PR_ALTA_CPEOLICA(idCProduccio,p_numAux,RSP);
        END IF;

        IF p_tipusCProd = 'SOLAR'
          THEN PR_ALTA_CPSOLAR(idCProduccio,p_numAux,RSP);
        END IF;

        IF p_tipusCProd = 'TERMICA'
          THEN PR_ALTA_CPTERMICA(idCProduccio,0,RSP);
        END IF;

        IF p_tipusCProd = 'CARBO'
          THEN
          PR_ALTA_CPCARBO(idCProduccio,0,RSP);

        END IF;

        COMMIT;
        RSP := 'OK';

EXCEPTION

	WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

       WHEN e_idCProd_negatiu THEN
        	RSP := 'ERROR: IDCPROD HA DE SER POSITIU';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

      WHEN e_dataFuncionament_erronia THEN
        	RSP := 'ERROR: DATA DE FUNCIONAMENT ' || p_dataFuncionament || ' SUPERIOR A LA ACTUAL '|| SYSDATE;
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

       WHEN e_dataDarreraInspecc_erronia THEN
        	RSP := 'ERROR: DATA  DARRERA INSPECCIO ' || p_dataDarreraInspecc || ' SUPERIOR A LA ACTUAL '|| SYSDATE;
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

       WHEN e_maxKWh_erroni THEN
        	RSP := 'ERROR: PARAMETRE MAX DE KWH AMB VALOR NEGATIU';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

        WHEN e_tipusCProd_erroni THEN
        	RSP := 'ERROR: TIPUS ('||p_tipusCProd||' DIFERENT DE ''EOLICA'', ''NUCLEAR'',''CARBO'',''SOLAR'' I ''TERMICA'' )';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

        WHEN e_numAux_erroni THEN
        	RSP := 'ERROR: PARAMETRE NUMAUX AMB VALOR NEGATIU';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
		ROLLBACK;

        WHEN DUP_VAL_ON_INDEX THEN
        	RSP := 'ERROR: CLAU IDCPROD (' || idCProduccio|| ') DUPLICADA A LA TAULA CENTRALPROD.';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
		ROLLBACK;

	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		ROLLBACK;

END;
/