CREATE OR REPLACE
PROCEDURE PR_ALTA_COMPTADOR (

p_idComptador IN comptador.idComptador%TYPE,
p_idModel IN comptador.idModel%TYPE,
p_adreca IN comptador.adreca%TYPE,
p_codiPostal IN comptador.codiPostal%TYPE,
p_localitat IN comptador.localitat%TYPE,
p_pais IN comptador.pais%TYPE,
p_consumAbsolut IN comptador.consumAbsolut%TYPE,
p_consumMensual IN comptador.consumMensual%TYPE,
p_anyFabricacio IN comptador.anyFabricacio%TYPE,

RSP OUT NOCOPY TAULALOG.RSP%TYPE
)

/*****************************************************************************
--  PROCEDIMENT:  PR_ALTA_COMPTADOR
--
--  Dona d'alta un comptador en la taula COMPTADOR.
-- 
--
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la
--  TAULALOG el resultat de la operaci.
--
--  ENTRADA : p_IDCOMPTADOR NUMBER(10), p_IDMODEL NUMBER(6), p_ADRECA VARCHAR2(30), 
--      p_CODIPOSTAL CHAR(5), p_LOCALITAT VARCHAR2(30), p_PAIS VARCHAR2(20), 
--      p_CONSUMABSOLUT NUMBER(8,1), p_CONSUMMENSUAL NUMBER(8,1), 
--      p_ANYFABRICACIO NUMBER(4) .
--
--  SORTIDA : RSP indicant si ha anat correcte o si ha hagut errors.
--
--  Si el parmetre d'entrada idComptador  = 0 s'agafa valor de la seqncia.
--  Data Baixa s'assignar nul.
--  Data Lectura s'assignar nul
--  Si s nou el comptador, el consum hauria de ser zero, per si s nou
--   noms el sistema, el consumMensual ser zero i el consumAbsolut
--   ser el que tingui en el moment d'alta (AIX NO SERIA ERROR).
--
--    RSP = OK : S'inserta registre.
--
--    RSP = ERROR + motiu, si hi ha algun error.
--
--    Motius error tractats:
--        - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA NO SUPORTATS.
--              (EXCEPTE dataLectura que quan es dona d'alta pot ser nul)
--        - ERROR:  IDMODEL DE COMPTADOR NO EXISTEIX A LA TAULA MODELCOMPT.
--	  - ERROR: CLAU IDCOMPTADOR  DUPLICADA A LA TAULA COMPTADOR.
--        - ERROR: CONSUM INTRODUIT AMB VALOR NEGATIU
--        - ERROR: ANY FABRICACIO MAJOR A L'ACTUAL.
--
******************************************************************************/


IS
    -- variables que s'insertaran a la taula COMPTADOR com a nuls
    dataBaixaCompt comptador.dataBaixa%TYPE := null;
    dataLecturaCompt comptador.dataLectura%TYPE := null;

    -- variables d'errors
    e_idmodel_noexisteix EXCEPTION;
    e_camps_nuls EXCEPTION;
    e_consum_erroni EXCEPTION;
    e_anyFabricacio_erroni EXCEPTION;

    -- variables insert taulaLOG
    nomProcediment taulaLog.procExecutat%TYPE := 'PR_ALTA_COMPTADOR';
    paramEntrada taulaLog.paramEntrada%TYPE := 'p_idComptador ' || p_idComptador || ', p_idModel: ' || p_idModel ||
            ', p_adreca : ' || p_adreca || ', p_codiPostal: ' || p_codiPostal || ', p_localitat: ' || p_localitat
            ||' , p_pais: '|| p_pais|| ', p_consumAbsolut: ' || p_consumAbsolut || ', p_consumMensual: '|| p_consumMensual
            ||' , p_anyFabricacio :'|| p_anyFabricacio;

    paramSortida taulaLog.paramSortida%TYPE := 'RSP';

    -- variables d's divers
    numElements number := 0;
    anyo number(4);
    numIdSeq number;

BEGIN

    -- VERIFIQUEM NULS ALS PARMETRES D'ENTRADA
    IF ((p_idComptador is null) or (p_idModel is null)  or (p_adreca is null) or (p_codiPostal is null)
         or (p_localitat is null) or (p_pais is null) or (p_consumAbsolut is null) or (p_consumMensual is null)
          or (p_anyFabricacio is null))
    THEN RAISE e_camps_nuls;
    END IF;

    -- VERIFIQUEM QUE EL MODEL DEL COMPTADOR EXISTEIXI A LA TAULA MODELCOMPT
    SELECT COUNT(*) INTO numElements  FROM MODELCOMPT  WHERE p_idModel = MODELCOMPT.idModel;
    IF numElements = 0
    THEN RAISE e_idModel_noexisteix;
    END IF;

    -- VERIFIQUEM QUE ELS CONSUMS INTRODUITS NO SIGUIN NEGATIUS
    IF p_consumAbsolut < 0 or p_consumMensual < 0
    THEN RAISE e_consum_erroni;
    END IF;

    --VERIFIQUEM QUE L'ANY DE FABRICACI NO SIGUI MAJOR A L'ACTUAL

    select TO_NUMBER(TO_CHAR(sysdate, 'yyyy')) into anyo from dual;
    IF p_anyFabricacio > anyo
    THEN RAISE e_anyFabricacio_erroni;
    END IF;

    -- SI NO HI HA ERRORS, INSERTEM A LA TAULA COMPTADOR
    -- si p_idComptador = 0, s'anota tamb el nmero de la seqncia seu 
    -- per saber amb quin idComptador ha sigut insertat.
        IF p_idComptador <> 0 THEN
    	INSERT INTO COMPTADOR VALUES (p_idComptador, p_idModel, p_adreca, p_codiPostal, p_localitat,
          p_pais, p_consumAbsolut, p_consumMensual,dataLecturaCompt, p_anyFabricacio, dataBaixaCompt);
        ELSE 
          INSERT INTO COMPTADOR VALUES (p_idComptador, p_idModel, p_adreca, p_codiPostal, p_localitat,
          p_pais, p_consumAbsolut, p_consumMensual,dataLecturaCompt, p_anyFabricacio, dataBaixaCompt)
          RETURNING IDcomptador INTO numIdSeq;
          ParamEntrada := '(numIDSeq :'||numIdSeq||') Params :'||paramEntrada;
        END IF;
        
	RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
        COMMIT;

	IF P_IDCOMPTADOR <> 0 THEN
	DBMS_OUTPUT.PUT_LINE('COMPTADOR ID '||P_IDCOMPTADOR||' AFEGIT A LA TAULA');
	ELSE	
        DBMS_OUTPUT.PUT_LINE('COMPTADOR ID '||numidSEq||' AFEGIT A LA TAULA');
	END IF;

EXCEPTION

	WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA NO SUPORTATS';
           	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;


        WHEN e_idModel_noexisteix THEN
        	RSP := 'ERROR:  IDMODEL DE COMPTADOR NO EXISTEIX A LA TAULA MODELCOMPT';
               	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

      WHEN e_consum_erroni THEN
        	RSP := 'ERROR: CONSUM INTRODUIT AMB VALOR NEGATIU';
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

        WHEN e_anyFabricacio_erroni THEN
        	RSP := 'ERROR: ANY FABRICACIO MAJOR A L''ACTUAL';
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
        	ROLLBACK;

        WHEN DUP_VAL_ON_INDEX THEN
        	RSP := 'ERROR: CLAU IDCOMPTADOR (' || p_idComptador|| ') DUPLICADA A LA TAULA COMPTADOR.';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
		ROLLBACK;

        WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		ROLLBACK;
END;
/