CREATE OR REPLACE
PROCEDURE PR_ALTA_CONNCOMPTADOR (

p_idComptador IN CONNCOMPTADOR.idComptador%TYPE ,
p_idCDist IN CONNCOMPTADOR.idCDist%TYPE ,
p_KWhConnexio CONNCOMPTADOR.KWhConnexio%TYPE,
p_tipusConn IN CONNCOMPTADOR.tipusConn%TYPE ,
p_estat IN CONNCOMPTADOR.estat%TYPE,

RSP OUT NOCOPY TAULALOG.RSP%TYPE

)

--/*****************************************************************************
--  PROCEDIMENT:  PR_ALTA_CONNCOMPTADOR
--    
--  Dona d'alta una connexi entre comptador i central de distribuci.
--  Aquesta taula relaciona un comptador i una Central de Distribuci 
--  i guarda informaci del tipus de connexi i si est activa o no.
--  
--  Un comptador noms pot rebre energia d'una c. Distribuci.
--  Noms  pot haver-hi una connexi activa i principal per cada parell
--     Comptador - Central Distribuci.
--  
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la 
--  TAULALOG el resultat de la operaci.
--  
--  ENTRADA : P_IDCOMPTADOR NUMBER(10), p_IDCDIST NUMBER(10), 
--        p_KWHCONNEXIO NUMBER(12,1), p_TIPUSCONN CHAR(1), p_ESTAT NUMBER(1) .
--
--  SORTIDA : RSP indicant si ha anat correcte o si ha hagut errors.
--  
--  El camp KWhConnexi ho donem d'alta com 0 i es recalcula amb els consums.  
--  KWhConnexi es reinicia anualment per la connexi.
--
--    RSP = OK : S'inserta registre.
--    
--    RSP = ERROR + motiu, si hi ha algun error.
--  
--    Motius Error :
--      - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA.
--      - ERROR: CLAU IDCOMPTADOR-IDCDIST DUPLICADA A LA TAULA CONNCOMPTADOR.
--  	- ERROR: IDCOMPTADOR  p_idComptador NO EXISTEIX A LA TAULA COMPTADOR.
--      - ERROR: IDCOMPTADOR  p_idComptador ESTA DE BAIXA A LA TAULA COMPTADOR.
--      - ERROR: IDCDIST  p_idCDist  NO EXISTEIX  A LA TAULA CENTRALDIST.
--      - ERROR: IDCDIST  p_idCDist  ESTA DE BAIXA A LA TAULA CENTRALDIST.
--      - ERROR: LA CONNEXIO ENTRE COMPTADOR I CENTRAL DE DISTRIBUCIO JA EXISTEIX.
--      - ERROR: PARAMETRE ESTAT  p_estat  INCORRECTE. VALORS ACCEPTATS (0,1)
--      - ERROR: PARAMETRE TIPUSCONN INCORRECTE. VALORS ACCEPTATS ('P','S')
--      - ERROR: EL COMPTADOR AMB IDCOMPTADOR p_idComptador JA TE UNA CONNEXIO ACTIVA.
--      - ERROR: EL COMPTADOR AMB IDCOMPTADOR p_idComptador JA TE UNA CONNEXIO PRINCIPAL
--      - ERROR: KWhCONNEXI HA DE SER ZERO AL DONAR D''ALTA.
--      - Altres.
--******************************************************************************/

IS

	-- variables errors
        e_camps_nuls EXCEPTION;
        e_idComptador_noexisteix EXCEPTION;
        e_idComptador_deBaixa EXCEPTION;
        e_idCDist_noexisteix EXCEPTION;
        e_idCdist_deBaixa EXCEPTION;
        e_connComptador_jaexisteix EXCEPTION;
        e_estat_erroni  EXCEPTION;
        e_tipusConn_erronia EXCEPTION;
        e_tipusConn_activa_existeix EXCEPTION;
        e_KWhConnexio_erronia EXCEPTION;
        e_tipusConn_principal_existeix EXCEPTION;
     
        -- variables insert taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_ALTA_CONNCOMPTADOR';
	paramEntrada taulaLog.paramEntrada%TYPE := 'p_idComptador: ' || p_idComptador || ', p_idCDist : ' || p_idCDist 
            || ', p_tipusConn : ' || p_tipusConn || ', p_estat : ' || p_estat  ;
            
	paramSortida taulaLog.paramSortida%TYPE := 'RSP';
  
        -- variable d's divers.
        numElements number :=0;       

BEGIN
               
                
      -- VERIFIQUEM ELS NULS DELS PARAMETRES D'ENTRADA
      IF  ((p_idComptador is null) or (p_idCDist is null) or ( p_tipusConn is null) or (p_estat is null) )
          THEN RAISE e_camps_nuls;
      END IF;
        
      -- VERIFIQUEM QUE EXISTEIX IDCOMPTADOR 
      SELECT COUNT(*) INTO numElements FROM COMPTADOR
       WHERE p_idComptador = COMPTADOR.idComptador ;
      IF numElements = 0 
        THEN RAISE e_idComptador_noexisteix;
      END IF;
           
    -- VERIFIQUEM QUE EXISTEIX IDCDIST .
    SELECT COUNT(*) INTO numElements FROM CENTRALDIST
      WHERE p_idCDist = CENTRALDIST.idCDist ;
    IF numElements = 0 
      THEN RAISE e_idCDist_noexisteix;
    END IF;  
    
    -- VERIFIQUEM IDCOMPTADOR NO ESTIGUI DE BAIXA 
      SELECT COUNT(*) INTO numElements FROM COMPTADOR
       WHERE p_idComptador = COMPTADOR.idComptador AND COMPTADOR.dataBaixa is NOT NULL;
      IF numElements > 0 
       THEN RAISE e_idComptador_deBaixa;
      END IF;
       
    -- VERIFIQUEM QUE  IDCDIST NO ESTIGUI DE BAIXA .
    SELECT COUNT(*) INTO numElements FROM CENTRALDIST
      WHERE p_idCDist = CENTRALDIST.idCDist AND CENTRALDIST.dataBaixa is NOT NULL;
    IF numElements > 0
      THEN RAISE e_idCDist_deBaixa;
    END IF;  
    
    -- VERIFIQUEM QUE LA CONNEXI NO EXISTEIXI
    SELECT COUNT(*) INTO numElements FROM CONNCOMPTADOR
      WHERE p_idComptador = CONNCOMPTADOR.idComptador and p_idCDist = CONNCOMPTADOR.idCDist;
    IF numElements <> 0 
      THEN RAISE  e_connComptador_jaexisteix;
    END IF;
    
    -- VERIFIQUEM QUE EL CAMP ESTAT SIGUI 0 o  1  (connexi inactiva o activa)
    IF p_estat <> 0 and p_estat <> 1
      THEN RAISE e_estat_erroni;
    END IF;
    
    -- VERIFIQUEM QUE TIPUSCONN SIGUI 'P' , 'S' 
    IF p_tipusConn <> 'P' AND p_tipusConn <>'S' 
      THEN RAISE e_tipusConn_erronia;
    END IF;
    
    -- VERIFIQUEM QUE NO EXISTEIXI UNA CONNEXIO ACTIVA PER AQUEST COMPTADOR
    SELECT COUNT(*) INTO numElements FROM CONNCOMPTADOR
        WHERE p_idComptador = CONNCOMPTADOR.idComptador and CONNCOMPTADOR.estat = p_estat and p_estat = 1;
    IF numElements <> 0
       THEN RAISE e_tipusConn_activa_existeix;
    END IF;
    
     -- VERIFIQUEM QUE AQUEST COMPTADOR NO TINGU UNA CONNEXI PRINCIPAL 
     -- (NOMsUNA CONNEXI DE COMPTADOR POT SER PRINCIPAL I LA RESTA SECUNDRIES (PER SI FALLA) 
    SELECT COUNT(*) INTO numElements FROM CONNCOMPTADOR
        WHERE p_idComptador = CONNCOMPTADOR.idComptador and CONNCOMPTADOR.tipusConn = 'P' and p_tipusConn = 'P';
    IF numElements <> 0
       THEN RAISE e_tipusConn_principal_existeix;
    END IF;
    
    -- VERIFIQUEM QUE PARMETRE KWHCONNEXIO S ZERO AL DONAR D'ALTA
    -- 
    IF p_KWhConnexio <> 0 THEN
        RAISE e_KWhConnexio_erronia;
    END IF;
    
    
    -- SI NO HI HA ERRORS, INSERTEM a CONNCOMPTADOR
    	INSERT INTO CONNCOMPTADOR VALUES (p_idComptador, p_idCDist, p_KWhConnexio, p_tipusConn, p_estat);
	RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
        COMMIT;
        
        DBMS_OUTPUT.PUT_LINE('CONNEXIO COMPTADOR-CENTRAL DIST '|| p_idComptador||'-'
                              ||p_idCDist||' DONADA D''ALTA');

EXCEPTION

	WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
   		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
       WHEN e_idComptador_noexisteix  THEN
        	RSP := 'ERROR: IDCOMPTADOR '|| p_idComptador|| ' NO EXISTEIX A LA TAULA COMPTADOR';
   		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		RollBack;
              
       WHEN e_idComptador_deBaixa  THEN
        	RSP := 'ERROR: IDCOMPTADOR '|| p_idComptador|| '  ESTA DE BAIXA A LA TAULA COMPTADOR';
   		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
	 
        WHEN e_idCDist_noexisteix  THEN
        	RSP := 'ERROR: IDCDIST '|| p_idCDist|| ' NO EXISTEIX  A LA TAULA CENTRALDIST';
   		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;        
                
         WHEN e_idCDist_deBaixa  THEN
        	RSP := 'ERROR: IDCDIST '|| p_idCDist|| ' ESTA DE BAIXA A LA TAULA CENTRALDIST';
   		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;  
                
         WHEN e_connComptador_jaexisteix  THEN
        	RSP := 'ERROR: LA CONNEXIO ENTRE COMPTADOR I CENTRAL DE DISTRIBUCIO JA EXISTEIX';
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;     
       
        WHEN  e_estat_erroni   THEN
        	RSP := 'ERROR: PARAMETRE ESTAT '|| p_estat ||' INCORRECTE. VALORS ACCEPTATS (0,1)';
    		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;       
       
        WHEN  e_tipusConn_erronia   THEN
        	RSP := 'ERROR: PARAMETRE TIPUSCONN '|| p_tipusConn ||' INCORRECTE. VALORS ACCEPTATS (''P'',''S'')';
    		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;       
       
         WHEN  e_tipusConn_activa_existeix   THEN
        	RSP := 'ERROR: EL COMPTADOR AMB IDCOMPTADOR '||p_idComptador||' JA TE UNA CONNEXIO ACTIVA' ;
    		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;        
        
          WHEN   e_tipusConn_principal_existeix   THEN
        	RSP := 'ERROR: EL COMPTADOR AMB IDCOMPTADOR '||p_idComptador||' JA TE UNA CONNEXIO PRINCIPAL' ;
    		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;       
          WHEN   e_KWhConnexio_erronia THEN
        	RSP := 'ERROR: KWhCONNEXI HA DE SER ZERO AL DONAR D''ALTA' ;
    		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;  
         
	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR. '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
        WHEN DUP_VAL_ON_INDEX THEN
                RSP := 'ERROR: CLAU IDCOMPTADOR-IDCDIST (' || p_idComptador|| '-'|| p_idCDist|| ') DUPLICADA A LA TAULA CONNCOMPTADOR';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
          
	WHEN OTHERS THEN
		RSP := 'ERROR SENSE TRACTAR PR_ALTA_COMPTADOR - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;

END;
/