CREATE OR REPLACE
PROCEDURE PR_ALTA_CONNLINIA (

p_idCDist IN CONNLINIA.idCDist%TYPE ,
p_idLinia IN CONNLINIA.idLinia%TYPE,
p_idCProd IN CONNLINIA.idCProd%TYPE,
p_tipusConn IN CONNLINIA.tipusConn%TYPE,
p_estat IN CONNLINIA.estat%TYPE,

RSP OUT NOCOPY TAULALOG.RSP%TYPE

)

--/*****************************************************************************
--  PROCEDIMENT:  PR_ALTA_CONNLINIA
--    
--  Dona d'alta una connexi entre Central Distribuci, lnia i C. Producci
--  i guarda informaci del tipus que s s (principal o secundria- reserva), 
--  i si est activa o no a la taula CONNLINIA.
--  
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la 
--  TAULALOG el resultat de la operaci.
--  
--  ENTRADA : P_IDCDIST NUMBER(10), p_IDLINIA NUMBER(10) , p_IDCPROD NUMBER(10),
--            p_TIPUSCONN   CHAR(1) , p_ESTAT  NUMBER(1). 
--
--  SORTIDA : RSP varchar2(300) indicant si ha anat correcte o hi ha errors.
-- 
--    RSP = OK : S'inserta registre.
--    
--    RSP = ERROR + motiu, si hi ha algun error.
--  
--    Motius Error :
--      - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA.
--      - ERROR: LA CONNEXIO idCDist-idLinia-idCprod JA EXISTEIX.
--  	- ERROR: IDCPROD p_idCProd NO EXISTEIX A LA TAULA CENTRALPROD.
--	- ERROR: IDLINIA p_idLinia NO EXISTEIX A LA TAULA LINIA.
--      - ERROR: IDCDIST p_idCDist NO EXISTEIX A LA TAULA CENTRALDIST.
--      - ERROR: IDCPROD p_idCProd ESTA DE BAIXA A LA TAULA CENTRALPROD.
--      - ERROR: IDLINIA  p_idLinia ESTA DE BAIXA A LA TAULA LINIA.
--      - ERROR: IDCDIST  p_idCDist ESTA DE BAIXA A LA TAULA CENTRALDIST
--      - ERROR: LA CONNEXIO idCDist-idCprod JA EXISTEIX (NO BENEFICIA LA ALTA DISPONIBILITAT)
--		(encara que sigui per diferents linies, es considera error
-- 		 ja que hauria d'estar connectada a una altra CProd per poder
--               beneficiar-se de la alta disponibilitat)
--      - ERROR: PARAMETRE ESTAT  p_estat INCORRECTE. VALORS ACCEPTATS (0,1).
--      - ERROR: PARAMETRE TIPUSCONN  p_tipusConn INCORRECTE. VALORS ACCEPTATS ('P','S').
--      - ERROR: LA CENTRAL DISTRIBUCIO AMB IDCDIST p_idCDIST JA TE UNA CONNEXIO ACTIVA.
--      - ERROR: LA CENTRAL DISTRIBUCIO AMB IDCDIST p_idCDIST JA TE UNA CONNEXIO PRINCIPAL.
--      - Altres.
--
--******************************************************************************/

IS
        -- variables errors
	e_camps_nuls EXCEPTION;
        e_idCDist_noexisteix EXCEPTION;
        e_idLinia_noexisteix EXCEPTION;
	e_idCProd_noexisteix EXCEPTION;
	e_idCDist_deBaixa EXCEPTION;
	e_idLinia_deBaixa EXCEPTION;
	e_idCProd_deBaixa EXCEPTION;
        e_centrals_ja_unides EXCEPTION;
        e_connLinia_jaexisteix EXCEPTION;
        e_estat_erroni  EXCEPTION;
        e_tipusConn_erronia EXCEPTION;
        e_tipusConn_activa_existeix EXCEPTION;
        e_tipusConn_principal_existeix EXCEPTION;
     
        -- variables insert taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_ALTA_CONNLINIA';
	paramEntrada taulaLog.paramEntrada%TYPE := 'p_idCDist: ' || p_idCDist || ', p_idLinia : ' || p_idLinia 
           || ', p_idCProd: ' || p_idCProd || ',  p_tipusConn : ' || p_tipusConn || ', p_estat : ' || p_estat  ;
            
	paramSortida taulaLog.paramSortida%TYPE := 'RSP';
  
      numElements number :=0 ;      

BEGIN
      -- VERIFIQUEM ELS NULS DELS PARAMETRES D'ENTRADA
      IF  ((p_idCProd is null)or (p_idLinia is null) or (p_idCDist is null) or 
          ( p_tipusConn is null) or (p_estat is null) )
        THEN RAISE e_camps_nuls;
      END IF;
           
    -- VERIFIQUEM QUE EXISTEIX IDCDIST 
    SELECT COUNT(*) INTO numElements FROM CENTRALDIST
      WHERE p_idCDist = CENTRALDIST.idCDist ;
    IF numElements = 0 
      THEN RAISE e_idCDist_noexisteix;
    END IF;  
    
    -- VERIFIQUEM QUE EXISTEIX IDCPROD 
    SELECT COUNT(*) INTO numElements FROM CENTRALPROD
       WHERE p_idCProd = CENTRALPROD.idCProd ;
    IF numElements = 0 
      THEN RAISE e_idCProd_noexisteix;
    END IF;
      
     -- VERIFIQUEM QUE EXISTEIX LA LINIA 
    SELECT COUNT(*) INTO numElements FROM LINIA
      WHERE p_idLinia = LINIA.idLinia ;
    IF numElements = 0 
       THEN RAISE e_idLinia_noexisteix;
    END IF;

    -- VERIFIQUEM QUE IDCDIST NO ESTIGUI DE BAIXA
    SELECT COUNT(*) INTO numElements FROM CENTRALDIST
      WHERE p_idCDist = CENTRALDIST.idCDist AND CENTRALDIST.dataBaixa is NOT NULL;
    IF numElements > 0 
      THEN RAISE e_idCDist_deBaixa;
    END IF;  
    
    -- VERIFIQUEM QUE IDCPROD NO ESTIGUI DE BAIXA
    SELECT COUNT(*) INTO numElements FROM CENTRALPROD
       WHERE p_idCProd = CENTRALPROD.idCProd AND CENTRALPROD.dataBaixa IS NOT NULL;
    IF numElements > 0 
      THEN RAISE e_idCProd_deBaixa;
    END IF;
      
     -- VERIFIQUEM QUE la LINIA NO ESTIGUI DE BAIXA
   SELECT COUNT(*) INTO numElements FROM LINIA
      WHERE p_idLinia = LINIA.idLinia AND LINIA.dataBAixa IS NOT NULL;
   IF numElements > 0 
     THEN RAISE e_idLinia_deBaixa;
   END IF;

   
   -- VERIFIQUEM QUE LA CONNEXI NO EXISTEIXI
    SELECT COUNT(*) INTO numElements FROM CONNLINIA
      WHERE  p_idCDist = CONNLINIA.idCDist and p_idLinia = CONNLINIA.idLinia and p_idCProd = CONNLINIA.idCProd;
    IF numElements <> 0 
      THEN RAISE  e_connLinia_jaexisteix;
    END IF;

   -- VERIFIQUEM QUE NO EXISTEIXI CONNEXIO ENTRE LA MATEIXA IDCDIST-IDCPROD
   -- ENCARA QUE SIGUI PER DIFERENTS LINIES
   -- (AIX BENEFICIA EL QUE HI HAGI ALTA DISPONIBILITAT, JA QUE S MILLOR QUE 
   -- LES C.DIST PENGIN DE DIFERENTS C.PROD)
      
    SELECT COUNT(*) INTO numElements FROM CONNLINIA
       WHERE p_idCDist = CONNLINIA.idCDist and p_idCProd = CONNLINIA.idCprod 
       AND CONNLINIA.tipusConn <> 'B';
    IF numElements > 0 
       THEN RAISE e_centrals_ja_unides;
     END IF;
      
    
    
    -- VERIFIQUEM QUE EL CAMP ESTAT SIGUI 0 - 1  (connexi inactiva o activa)
    IF p_estat <> 0 and p_estat <> 1
      THEN RAISE e_estat_erroni;
    END IF;
    
    -- VERIFIQUEM QUE TIPUSCONN SIGUI 'P'o'S' 
    IF p_tipusConn <> 'P' AND p_tipusConn <>'S' 
      THEN RAISE e_tipusConn_erronia;
    END IF;
    
    
    -- VERIFIQUEM QUE NO EXISTEIXI UNA CONNEXIO ACTIVA PER AQUESTA CENTRAL DISTRIBUCI
    -- EN CAS QUE EL COMPTADOR PUGUI REBRE ENERGIA DE MS D'UNA CENTRAL DE DIST.,
    -- AQUESTA VERIFICACI S'ESBORRARIA.
    -- EN AQUEST TFC ES SUPOSA QUE NOMS REP D'UNA LINIA I LES ALTRES ESTAN EN RESERVA
    
    SELECT COUNT(*) INTO numElements FROM CONNLINIA
        WHERE p_idCDist = CONNLINIA.idCDist and CONNLINIA.estat = p_estat and p_estat = 1;
    IF numElements <> 0
       THEN RAISE e_tipusConn_activa_existeix;
    END IF;
    
     -- VERIFIQUEM QUE AQUESTA CENTRAL DISTRIBUCI NO TINGUI JA UNA CONNEXI PRINCIPAL 
     -- (NOMS UNA CONNEXI POT SER PRINCIPAL I LA RESTA SECUNDRIES PER SI FALLA) 
    SELECT COUNT(*) INTO numElements FROM CONNLINIA
        WHERE p_idCDist = CONNLINIA.idCDist and CONNLINIA.tipusConn = 'P' and p_tipusConn = 'P';
    IF numElements <> 0
       THEN RAISE e_tipusConn_principal_existeix;
    END IF;
 
  
     -- SI NO HI HA ERRORS, INSERTEM 
       
	INSERT INTO CONNLINIA VALUES (p_idCDist, p_idLinia, p_idCProd,p_tipusConn, p_estat);
	
        RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
        
        COMMIT;
        
        DBMS_OUTPUT.PUT_LINE('CONNEXIO AMB CLAU ('||p_idCDist||'-'||p_idLinia||'-'||p_idCProd||') AFEGIDA');
       
EXCEPTION

	WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
 		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
         	Rollback;
                
       WHEN e_idCProd_noexisteix  THEN
        	RSP := 'ERROR: IDCPROD '|| p_idCProd|| ' NO EXISTEIX A LA TAULA CENTRALPROD';
 		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

	WHEN e_idLinia_noexisteix  THEN
        	RSP := 'ERROR: IDLINIA '|| p_idLinia|| ' NO EXISTEIX A LA TAULA LINIA';
 		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
                Rollback;      
	
        WHEN e_idCDist_noexisteix  THEN
        	RSP := 'ERROR: IDCDIST '|| p_idCDist|| ' NO EXISTEIX A LA TAULA CENTRALDIST';
 		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;    
       
        WHEN e_idCProd_deBaixa  THEN
        	RSP := 'ERROR: IDCPROD '|| p_idCProd|| ' ESTA DE BAIXA A LA TAULA CENTRALPROD';
 		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
                Rollback;

	WHEN e_idLinia_deBaixa  THEN
        	RSP := 'ERROR: IDLINIA '|| p_idLinia|| '  ESTA DE BAIXA A LA TAULA LINIA';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
         	Rollback;      
	
        WHEN e_idCDist_deBaixa  THEN
        	RSP := 'ERROR: IDCDIST '|| p_idCDist|| ' ESTA DE BAIXA A LA TAULA CENTRALDIST';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
                Rollback;           

	 WHEN e_centrals_ja_unides THEN
        	RSP := 'ERROR: LA CONNEXIO idCDist-idCprod ('|| p_idCdist ||'-'|| p_idCProd ||') JA EXISTEIX (NO BENEFICIA LA ALTA DISPONIBILITAT)';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
          	Rollback;     
               
        WHEN e_connLinia_jaexisteix  THEN
        	RSP := 'ERROR: LA CONNEXIO idCDist-idLinia-idCprod ('|| p_idCdist || '-'|| p_idLinia||'-'|| p_idCProd ||') JA EXISTEIX';
       		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
	  	Rollback;     
       
        WHEN  e_estat_erroni   THEN
        	RSP := 'ERROR: PARAMETRE ESTAT '|| p_estat ||' INCORRECTE. VALORS ACCEPTATS (0,1)';
       		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
         	Rollback;       
       
        WHEN  e_tipusConn_erronia   THEN
        	RSP := 'ERROR: PARAMETRE TIPUSCONN '|| p_tipusConn ||' INCORRECTE. VALORS ACCEPTATS (''P'',''S'')';
       		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
        	Rollback;       
       
         WHEN  e_tipusConn_activa_existeix   THEN
        	RSP := 'ERROR: LA CENTRAL DISTRIBUCIO AMB IDCDIST '||p_idCDIST||' JA TE UNA CONNEXIO ACTIVA' ;
       		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
         	Rollback;        
        
          WHEN   e_tipusConn_principal_existeix   THEN
        	RSP := 'ERROR: LA CENTRAL DISTRIBUCIO AMB IDCDIST '||p_idCDIST||' JA TE UNA CONNEXIO PRINCIPAL' ;
       		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
        	Rollback;       
         
	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
        WHEN DUP_VAL_ON_INDEX THEN
                RSP := 'ERROR: CLAU DUPLICADA A LA TAULA CONNLINIA';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
          
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR PR_ALTA_CONNLINIA - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;

END;
/