CREATE OR REPLACE
PROCEDURE PR_ALTA_CPCARBO (

p_idCProd IN CPCARBO.idCProd%TYPE,
p_KGEmissions IN CPCARBO.KGEmissions%TYPE,
RSP OUT NOCOPY taulalog.RSP%TYPE

)

--/*****************************************************************************
--  PROCEDIMENT:  PR_ALTA_CPCARBO
--    
--  Rep els parmetre des de PR_ALTA_CENTRALPROD , i si aquella
--  s del tipus CARBO llavors crida a aquest procediment, passant el idCProd i
--  els Kg d'emissions que al ser de nova incorporaci, ser zero.   
--  
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la 
--  TAULALOG el resultat de la operaci
--    
--  ENTRADA : P_IDCPROD  NUMBER(10) , p_KGSEMISSIONS   NUMBER(12,1) .
--
--  SORTIDA: RSP VARCHAR(300) indicant si ha anat correcte o si ha hagut errors.
--         
--   RSP = OK : S'inserta registre a la taula CPCARBO.
--    
--    RSP = ERROR + motiu, si hi ha algun error.
--  
--    Motius Error :
--      - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA.
--      - ERROR: PARAMETRE KGEMISSIONS AMB VALOR NEGATIU.
--      - ERROR: CLAU IDCPROD DUPLICADA A LA TAULA CPCARBO.
--      - Altres
--
--******************************************************************************/



IS
	-- variables errors
        e_camps_nuls EXCEPTION;
        e_KGEmissions_erroni  EXCEPTION;
        
        -- variables insert taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_ALTA_CPCARBO';
	paramEntrada taulaLog.paramEntrada%TYPE := 'idCProd: ' || p_idCProd || 'KgEmissions : ' || p_KGEmissions;
	paramSortida taulaLog.paramSortida%TYPE := 'RSP';

BEGIN
                
      -- VERIFIQUEM ELS NULS DELS PARAMETRES D'ENTRADA
      IF   (p_KgEmissions is null)
          THEN RAISE e_camps_nuls;
      END IF;
    
     -- VERIFIQUEM QUE MAXKWH ES UNA QUANTITAT POSITIVA
     IF p_Kgemissions < 0 
      THEN RAISE e_KgEmissions_erroni;
     END IF;
    
     -- SI NO HI HA ERRORS, INSERTEM 
    INSERT INTO CPCARBO VALUES (p_idCProd, p_KGEmissions);
    RSP := 'OK';
    PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
         
    COMMIT;
      
    DBMS_OUTPUT.PUT_LINE('OK. REGISTRE AFEGIT A LA TAULA: CPCARBO. IDCPROD ('|| p_idCprod|| ').');
    
EXCEPTION

	WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                  
       WHEN e_KGEmissions_erroni THEN
        	RSP := 'ERROR: PARAMETRE KGEMISSIONS AMB VALOR NEGATIU';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
               
	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
        WHEN DUP_VAL_ON_INDEX THEN
        	RSP := 'ERROR: CLAU IDCPROD (' || p_idCProd|| ') DUPLICADA A LA TAULA CPCARBO.';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;

	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR A PR_ALTA_CPCARBO - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;
	
END;
/