CREATE OR REPLACE
PROCEDURE PR_ALTA_CPEOLICA (

p_idCProd IN CPEOLICA.idCProd%TYPE,
p_numMolins IN CPEOLICA.numMolins%TYPE,

RSP OUT NOCOPY TAULALOG.RSP%TYPE

)

--/*****************************************************************************
--  PROCEDIMENT:  PR_ALTA_CPEOLICA
--    
--  Rep els parmetres des de PR_ALTA_CENTRALPROD i si aquella
--  s del tipus EOLICA llavors crida a aquest procediment, passant el idCProd, 
--  i els numeros de molins.
--   
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la 
--  TAULALOG el resultat de la operaci.
--  
--  ENTRADA : P_ idCProd  NUMBER(10) ,  p_NUMMOLINS NUMBER(6) .
--
--  SORTIDA: RSP VARCHAR2(300)indicant si ha anat correcte o si ha hagut errors.
--   
--   RSP = OK : S'inserta registre a la taula CPEOLICA.
--    
--   RSP = ERROR + motiu, si hi ha algn error.
--  
--    Motius errors tractats :
--      - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA.
--      - ERROR: PARAMETRE NUMMOLINS AMB VALOR NEGATIU.
--      - ERROR: CLAU IDCPROD  p_idCProd DUPLICADA A LA TAULA CPEOLICA.
--      - Altres
--
--******************************************************************************/


IS
	-- variables errors
        e_camps_nuls EXCEPTION;
	e_numMolins_erroni EXCEPTION;
        
        -- variables insert taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_ALTA_CPEOLICA';
	paramEntrada taulaLog.paramEntrada%TYPE := 'p_idCProd: ' || p_idCProd || 'p_numMolins: '|| p_numMolins;
	paramSortida taulaLog.paramSortida%TYPE := 'RSP';
   
BEGIN
      -- VERIFIQUEM ELS NULS DELS PARAMETRES D'ENTRADA
    IF   (p_idCProd is null) or  (p_numMolins is null)
      THEN RAISE e_camps_nuls;
    END IF;
    
     -- VERIFIQUEM QUE p_numMolins ES UNA QUANTITAT POSITIVA
    IF p_numMolins < 0 
      THEN RAISE e_numMolins_erroni;
    END IF;

       -- SI NO HI HA ERRORS, INSERTEM 
    INSERT INTO CPEOLICA VALUES (p_idCProd, p_numMolins);
    RSP := 'OK';
    PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
      
    COMMIT;
        
    DBMS_OUTPUT.PUT_LINE('OK. REGISTRE AFEGIT A LA TAULA: CPEOLICA. IDCPROD ('|| p_idCProd|| ').');
    
EXCEPTION

      WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                  
      WHEN e_numMolins_erroni THEN
        	RSP := 'ERROR: PARAMETRE NUMMOLINS AMB VALOR NEGATIU';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
               
      WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
        WHEN DUP_VAL_ON_INDEX THEN
        	RSP := 'ERROR: CLAU IDCPROD (' || p_idCProd|| ') DUPLICADA A LA TAULA CPEOLICA.';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
      
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR A PR_ALTA_CPEOLICA - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;
	
END;
/