CREATE OR REPLACE
PROCEDURE PR_ALTA_CPNUCLEAR (

p_idCProd IN CPNUCLEAR.idCProd%TYPE,
p_energiaMin IN CPNUCLEAR.energiaMin%TYPE,
p_KgRebuig IN CPNUCLEAR.KgRebuig%TYPE,
RSP OUT NOCOPY TAULALOG.RSP%TYPE

)

--/*****************************************************************************
--  PROCEDIMENT:  PR_ALTA_CPNUCLEAR
--    
--  Rep els parmetres des de PR_ALTA_CENTRALPROD i si aquella
--  s del tipus NUCLEAR llavors crida a aquest procediment, passant el idCProd, 
--  i la energia mnima i els kg de rebuig.
--
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la 
--  TAULALOG el resultat de la operaci.
--  
--  ENTRADA: P_IDCPROD NUMBER(10), p_ENERGIAMIN NUMBER(12,1),
--            p_KGSREBUIG  NUMBER(12,1). 
--
--  SORTIDA : RSP VARCHAR2(300) indicant si ha anat correcte o si ha hagut errors.
--  
--    RSP = OK : S'inserta registre a la taula CPNUCLEAR.
--    
--    RSP = ERROR + motiu, si hi ha algn error.
--  
--    Motius errors tractats :
--      - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA.
--      - ERROR: PARAMETRE KGREBUIG AMB VALOR NEGATIU.
--      - ERROR: PARAMETRE ENERGIA MNIMA AMB VALOR NEGATIU.
--      - ERROR: CLAU IDCPROD  p_idCProd DUPLICADA A LA TAULA CPNUCLEAR.
--      - Altres
--
--******************************************************************************/


IS
	-- variables errors
        e_camps_nuls EXCEPTION;
        e_KGRebuig_erroni  EXCEPTION;
	e_energiaMin_erronia EXCEPTION;
        
        -- variables insert taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_ALTA_CPNUCLEAR';
	paramEntrada taulaLog.paramEntrada%TYPE := 'p_idCProd : ' || p_idCProd || ', p_energiaMin.: '|| p_energiaMin 
		|| ', p_KGRebuig : ' || p_KGRebuig;
            
	paramSortida taulaLog.paramSortida%TYPE := 'RSP';
	
BEGIN
      -- VERIFIQUEM ELS NULS DELS PARAMETRES D'ENTRADA
    IF   (p_idCProd is null) or  (p_KgRebuig is null) or (p_energiaMin is null)
      THEN RAISE e_camps_nuls;
    END IF;
    
     -- VERIFIQUEM QUE KG DE REBUIG ES UNA QUANTITAT POSITIVA
    IF p_KgRebuig < 0 
      THEN RAISE e_KgRebuig_erroni;
    END IF;
    
    -- VERIFIQUEM QUE ENERGIA MINIMA NO SIGUI NEGATIVA
    IF p_energiaMin < 0
      THEN RAISE e_energiaMin_erronia;
    END IF;
      
      
    -- SI NO HI HA ERRORS, INSERTEM 
    INSERT INTO CPNUCLEAR VALUES (p_idCProd, p_energiaMin, p_KGRebuig);
    RSP := 'OK';
    PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
    
    COMMIT;
        
    DBMS_OUTPUT.PUT_LINE('OK. REGISTRE AFEGIT A LA TAULA: CPNUCLEAR. IDCPROD ('|| p_idCProd|| ').');
    
EXCEPTION

      WHEN e_camps_nuls THEN
                RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
          	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                  
      WHEN e_KGRebuig_erroni THEN
        	RSP := 'ERROR: PARAMETRE KGREBUIG AMB VALOR NEGATIU';
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
               
     WHEN e_energiaMin_erronia  THEN
        	RSP := 'ERROR: PARAMETRE ENERGIA MNIMA AMB VALOR NEGATIU';
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
      WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
      WHEN DUP_VAL_ON_INDEX THEN
        	RSP := 'ERROR: CLAU IDCPROD (' || p_idCProd|| ') DUPLICADA A LA TAULA CPNUCLEAR.';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
      
      WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR A PR_ALTA_CPNUCLEAR - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;

END;
/