CREATE OR REPLACE
PROCEDURE PR_ALTA_CPSOLAR (

p_idCProd IN CPSOLAR.idCProd%TYPE,
p_numPanels IN CPSOLAR.numPanels%TYPE,

RSP OUT NOCOPY TAULALOG.RSP%TYPE

)

--/*****************************************************************************
--  PROCEDIMENT:  PR_ALTA_CPSOLAR
--    
--  Rep els parmetres des de PR_ALTA_CENTRALPROD, i si aquella
--  s del tipus SOLAR llavors crida a aquest procediment, passant el idCProd, 
--  i els numeros de Panels.
--
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la 
--  TAULALOG el resultat de la operaci
--  
--  ENTRADA : P_IDCPROD NUMBER(10), p_ NUMPANELS NUMBER(6). 
--
--  SORTIDA : RSP VARCHAR2(300) indicant si ha anat correcte o  amb errors.
--
--    RSP = OK : S'inserta registre a la taula CPSOLAR.
--    
--    RSP = ERROR + motiu, si hi ha algn error.
--  
--    Motius Error :
--      - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA.
--      - ERROR: PARAMETRE NUMPanels AMB VALOR NEGATIU.
--      - ERROR: CLAU IDCPROD p_idCProd  DUPLICADA A LA TAULA CPSOLAR.
--      - Altres
--
--******************************************************************************/


IS
	-- variables error
        e_camps_nuls EXCEPTION;
	e_numPanels_erroni EXCEPTION;
        
        -- variables insert taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_ALTA_CPSOLAR';
	paramEntrada taulaLog.paramEntrada%TYPE := ' p_idCProd: ' || p_idCProd || ', p_numPanels: '|| p_numPanels;
	paramSortida taulaLog.paramSortida%TYPE := 'RSP';
   
BEGIN
      -- VERIFIQUEM ELS NULS DELS PARAMETRES D'ENTRADA
    IF   (p_idCProd is null) or  (p_numPanels is null)
      THEN RAISE e_camps_nuls;
    END IF;
    
     -- VERIFIQUEM QUE p_numPanels ES UNA QUANTITAT POSITIVA
    IF p_numPanels < 0 
      THEN RAISE e_numPanels_erroni;
    END IF;
      
    -- SI NO HI HA ERRORS, INSERTEM 
	INSERT INTO CPSOLAR VALUES (p_idCProd, p_numPanels);
	RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);

        COMMIT;
        DBMS_OUTPUT.PUT_LINE('OK. REGISTRE AFEGIT A LA TAULA: CPSOLAR. IDCPROD ('|| p_idCProd|| ').');
    
EXCEPTION

      WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                  
      WHEN e_numPanels_erroni THEN
        	RSP := 'ERROR: PARAMETRE NUMPanels AMB VALOR NEGATIU';
      		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
               
      WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
      WHEN DUP_VAL_ON_INDEX THEN
        	RSP := 'ERROR: CLAU IDCPROD (' || p_idCProd|| ') DUPLICADA A LA TAULA CPSOLAR.';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
      
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR A PR_ALTA_CPSOLAR - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;

END;
/