CREATE OR REPLACE
PROCEDURE PR_ALTA_CPTERMICA(

p_idCProduccio IN CPTERMICA.idCProd%TYPE,
p_KGEmissions IN CPTERMICA.KGEmissions%TYPE,
RSP OUT NOCOPY TAULALOG.RSP%TYPE
)

--/*****************************************************************************
--  PROCEDIMENT:  PR_ALTA_CPTERMICA
--    
--  Rep els parmetres al donar d'alta una central de producci, i si aquella
--  s del tipus TERMICA llavors crida a aquest procediment, passant el idCProd i
--  els Kg d'emissions que al ser de nova incorporaci, ser zero.
-- 
--   Si una Central de Producci s del tipus TERMICA, tindr aquest valor al 
--  camp tipus de la taula CENTRALPROD, i s'uneix amb aquesta taula TERMICA amb
--  el idCProd 
-- 
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la 
--  TAULALOG el resultat de la operaci.
--    
--  ENTRADA : idCProd (number), KgEmissions (number)
--
--  SORTIDA : RSP indicant si ha anat correcte o si ha hagut errors.
-- 
--    RSP = OK : S'inserta registre a la taula CPTERMICA.
--    
--    RSP = ERROR + motiu, si hi ha algn error.
--  
--    Motius Error :
--      - Camps nuls.
--      - KGEmissions negatius.
--      - Clau idCProd duplicada.
--      - Altres.
--
--******************************************************************************/

IS
	-- variables errors
        e_camps_nuls EXCEPTION;
        e_KGEmissions_erroni  EXCEPTION;
        
        -- variables insert taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_ALTA_CPTERMICA';
	paramEntrada taulaLog.paramEntrada%TYPE := 'idCProd: ' || p_idCProduccio || 'KgEmissions : ' || p_KGEmissions;
	paramSortida taulaLog.paramSortida%TYPE := 'RSP';

BEGIN
                
      -- VERIFIQUEM ELS NULS DELS PARAMETRES D'ENTRADA
    IF   (p_KgEmissions is null)
      THEN RAISE e_camps_nuls;
    END IF;
      
     -- VERIFIQUEM QUE KGEMISSIONS ES UNA QUANTITAT POSITIVA
    IF p_Kgemissions < 0 
      THEN RAISE e_KgEmissions_erroni;
    END IF;
        
    -- SI NO HI HA ERRORS, INSERTEM 
	INSERT INTO CPTERMICA VALUES (p_idCProduccio, p_KGEmissions);
	RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
    
        COMMIT;
        DBMS_OUTPUT.PUT_LINE('OK. REGISTRE AFEGIT A LA TAULA: CPTERMICA. IDCPROD ('|| p_idCProduccio|| ').');
    
EXCEPTION

      WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                  
      WHEN e_KGEmissions_erroni THEN
        	RSP := 'ERROR: PARAMETRE KGEMISSIONS AMB VALOR NEGATIU';
      		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
               
      WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR. '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
      WHEN DUP_VAL_ON_INDEX THEN
        	RSP := 'ERROR: CLAU IDCPROD (' || p_idCProduccio|| ') DUPLICADA A LA TAULA CPTERMICA.';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
      
      WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR A PR_ALTA_CPTERMICA - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;
	
END;
/