CREATE OR REPLACE
PROCEDURE PR_ALTA_LINIA (

p_idLinia IN LINIA.idLinia%TYPE,
p_maxKWh IN LINIA.maxKWh%TYPE,

RSP OUT NOCOPY TAULALOG.RSP%TYPE

)

--/*****************************************************************************
--  PROCEDIMENT:  PR_ALTA_LINIA
--
--  Dona d'alta una una linia.

--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la
--  TAULALOG el resultat de la operaci.
--
--  ENTRADA : P_IDLINIA  NUMBER(10) ,  p_MAXKWH   NUMBER(12,1). 
--
--  SORTIDA : RSP varchar2(300) indicant si ha anat correcte o  errors.
--
--  Si el parmetre idLinea = 0, agafa el valor de segent de la seqncia
--  DataBaixa s'assignar nul al donar d'alta una nova linia.
--
--    RSP = OK : S'inserta registre a la taula LINIA.
--
--    RSP = ERROR + motiu, si hi ha algn error.
--
--    Motius Error :
--      - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA.
--      - ERROR: KWh amb valor negatiu.
--      - ERROR: CLAU IDLINIA p_idLinia DUPLICADA A LA TAULA LINIA.
--      - Altres.
--
--******************************************************************************/


IS
        -- variable que s'insertar a la taula com a nul.
        dataBaixaLinia LINIA.dataBaixa%TYPE := null;

        -- variables errors
        e_camps_nuls EXCEPTION;
        e_maxKWh_negatiu EXCEPTION;

        -- variables insert taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_ALTA_LINIA';
	paramEntrada taulaLog.paramEntrada%TYPE := 'p_idLinia: ' || p_idLinia || ',  p_maxKWh: ' || p_maxKWh;
        paramSortida taulaLog.paramSortida%TYPE := 'RSP';

        -- variables d's divers. Comptadors, etc.
        numElements number :=0;
        numIdSeq number;

BEGIN

      -- VERIFIQUEM ELS NULS DELS PARAMETRES D'ENTRADA
      IF  ((p_idLinia is null) or (p_maxKWh is null) )
          THEN RAISE e_camps_nuls;
      END IF;

     -- VERIFIQUEM QUE MAXKWH NO SIGUI NEGATIU
      IF p_maxKWh < 0
        THEN RAISE e_maxKWh_negatiu;
      END IF;

       -- SI NO HI HA ERRORS, INSERTEM,
       -- si p_idLinia = 0, anotem al log el nmero de seqncia amb el 
       -- que es dna d'alta com idLinia.
       
        IF p_idLinia <> 0 THEN

	INSERT INTO LINIA VALUES (p_idLinia, p_maxKWh, dataBaixaLinia);
        
        ELSE
        INSERT INTO LINIA VALUES (p_idLinia, p_maxKWh, dataBaixaLinia)
          RETURNING idLinia INTO numIdSeq;
        ParamEntrada := '(numIdSeq :'||numIdSeq||') Params :'||paramEntrada;
        END IF;
        
	RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
        COMMIT;
        IF P_IDLINIA <> 0 THEN
	DBMS_OUTPUT.PUT_LINE('REGISTRE AFEGIT A LA TAULA LINIA. IDLINIA: '|| p_idLinia );
	ELSE
		DBMS_OUTPUT.PUT_LINE('REGISTRE AFEGIT A LA TAULA LINIA. IDLINIA: '|| numIdSeq );
	END IF;

EXCEPTION

	WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

        WHEN e_maxKWh_negatiu THEN
        	RSP := 'ERROR: KWh amb valor negatiu';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;


        WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
		ROLLBACK;

        WHEN DUP_VAL_ON_INDEX THEN
                RSP := 'ERROR: CLAU IDLINIA (' || p_idLinia||') DUPLICADA A LA TAULA LINIA';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
		ROLLBACK;

        WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR - PR_ALTA_LINIA '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		ROLLBACK;

END;
/