CREATE OR REPLACE
PROCEDURE PR_ALTA_MODELCOMPT (

        p_idModel IN MODELCOMPT.idModel%TYPE,
	p_fabricant IN MODELCOMPT.fabricant%TYPE,
	p_modelCompt IN MODELCOMPT.modelCompt%TYPE,
	p_maxKW IN MODELCOMPT.maxKW%TYPE,
        
        RSP OUT NOCOPY TAULALOG.RSP%TYPE

/*****************************************************************************
--  PROCEDIMENT :  PR_ALTA_MODELCOMPT
--    
--  Dona d'alta un model de comptador en la taula MODELCOMPT.
--  Cada model t un mxim de potncia a la que pot funcionar.
--  
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la 
--  TAULALOG el resultat de la operaci.
--    
--  ENTRADA : p_IDMODEL NUMBER(6), p_FABRICANT VARCHAR2(20), p_MODELCOMPT VARCHAR2(15),
--    p_MAXKW  NUMBER (4,1) : mxima potncia que suporta el model de comptador.
--
--  SORTIDA : RSP VARCHAR2(300) indicant si ha anat correcte o si ha hagut errors.

--  
--  Si el parmetre d'entrada idModel = 0, s'agafa el valor de la seqncia.
--  
--    RSP = OK : S'inserta registre a la taula MODELCOMPT.
--    
--    RSP = ERROR + motiu, si hi ha algn error.
--  
--   Motius error tractats:
--        - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA.
--        - ERROR: CLAU IDMODEL p_idModel DUPLICADA A LA TAULA MODELCOMPT.
--        - ERROR: MAXKW HA DE SER UNA QUANTITAT POSITIVA.
--        - altres
--
******************************************************************************/

	
        )
	
IS
        -- variables errors
        e_camps_nuls EXCEPTION;
        e_maxKW_erroni EXCEPTION;
                       
	-- variables insert taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_ALTA_MODELCOMPT';
	paramEntrada taulaLog.paramEntrada%TYPE := 'P_idModel: ' || p_idModel || ', p_fabricant: ' || p_fabricant
		||' , p_modelCompt: '|| p_modelCompt|| ', p_maxKW: ' || p_maxKW;
        paramSortida taulaLog.paramSortida%TYPE := 'RSP';

	-- variable per guardar el sum de la seqncia
	numIDSeq number;
	
BEGIN
       
       -- VERIFIQUEM QUE NO HI HAGI PARMETRES NULS
        IF ((p_idModel is null)  or (p_fabricant is null) or (p_modelCompt is null)or(p_maxKW is null))
          THEN RAISE e_camps_nuls;
        END IF;
       
        -- VERIFIQUEM QUE EL PARMETRE MAXKW SIGUI POSITIU
        IF P_MAXKW < 0 THEN
            RAISE e_maxKW_erroni;
        END IF;
        
        -- SI NO HI HA ERRORS, INSERTEM 
	IF p_idModel <> 0 THEN
	INSERT INTO MODELCOMPT VALUES (p_idModel, p_fabricant, p_modelCompt,p_maxKW);
	ELSE
	INSERT INTO MODELCOMPT VALUES (p_idModel, p_fabricant, p_modelCompt,p_maxKW)
		RETURNING IDMODEL INTO NUMIDSEQ;
		ParamEntrada := '(numIdSeq :'||numIdSeq||') Params :'||paramEntrada;
	END IF;
	RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
	
        COMMIT;
	
	IF p_idModel <> 0 THEN
	DBMS_OUTPUT.PUT_LINE('MODEL COMPTADOR ID '|| p_idModel||' AFEGIT A TAULA MODELCOMPT');
	ELSE
	DBMS_OUTPUT.PUT_LINE('MODEL COMPTADOR ID '|| numIdsEQ||' AFEGIT A TAULA MODELCOMPT');
	END IF;
    
EXCEPTION
      
      WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
    		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
      	 WHEN e_maxKW_erroni THEN
        	RSP := 'ERROR: MAXKW HA DE SER UNA QUANTITAT POSITIVA';
    		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
        
        WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
        WHEN DUP_VAL_ON_INDEX THEN
        	RSP := 'ERROR: CLAU IDMODEL (' || p_idModel|| ') DUPLICADA A LA TAULA MODELCOMPT.';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
          
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR P_ALTAMODELCOMPT - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
                ROLLBACK;
	
END;
/