CREATE OR REPLACE
PROCEDURE PR_BAIXA_CENTRALDIST (

p_idCdist IN CENTRALDIST.idCDist%TYPE,
p_data IN CENTRALDIST.dataFuncionament%TYPE,

RSP OUT  NOCOPY TAULALOG.RSP%TYPE
)

/*****************************************************************************
--  PROCEDIMENT:  P_BAIXA_DENTRALDIST
--    
--  Dona de baixa una central de distribuci en la taula CENTRALDIST.
--  
--  Tant si hi ha errors com si no, s'afegeix a la 
--  TAULALOG el resultat de la operaci
--    
--  ENTRADA : P_IDCDIST  NUMBER(10), p_DATABAIXA DATE .
--  SORTIDA : RSP VARCHAR2(300) indicant si ha anat correcte o si ha hagut errors.
--  
--  si parmetre DataBAixa rebut s null, s'agafa del sistema.
--  
--    RSP = OK
--          - Es posa la data de baixa a Central Dist. 
--          - Es posa de baixa la connexi CONNCOMPTADOR on participi C.Dist.
--          - Es posa de baixa la connexi CONNLINIA on participi C.Dist.
--    
--   RSP = ERROR + motiu, si hi ha algn error.
--  
--    Motius Error :
--       - ERROR: ID CENTRAL DIST. p_idCDist NO EXISTEIX O JA ESTA DONAT DE BAIXA.
--       - ERROR: PARAMETRE IDCDIST NUL. 
--       - ERROR: DATA BAIXA HA DE SER IGUAL O MAJOR A LA DATA DE FUNCIONAMENT.
--       - ERROR: ID CENTRAL DISTRIBUCIO p_idCDist SUBMINISTRA A COMPTADORS.
--       - Altres.
--    
--
******************************************************************************/

IS
    -- variables errors
    e_idCDist EXCEPTION;
    e_idCDist_nul EXCEPTION;
    e_dataBaixa_erronia EXCEPTION;
    e_subministra_Comptadors EXCEPTION;
       
    -- variables insert taulaLOG
    nomProcediment taulaLog.procExecutat%TYPE := 'PR_BAIXA_CENTRALDIST';
    paramEntrada taulaLog.paramEntrada%TYPE := 'p_idCDist: ' || p_idCDist || ', p_data:'|| p_data;
    paramSortida taulaLog.paramSortida%TYPE := 'RSP';

    -- variable que s'insertar a la taula CENTRALDIST amb la data de baixa.
    dataBaixaCDist  CENTRALDIST.dataFuncionament%TYPE;
    
    -- variable on es guarda la data de funcionament de la central de distribuci.
    t_dataFuncionament  CENTRALDIST.dataFuncionament%TYPE;
     
    -- variables d's divers. Comptadors.
    numElements number := 0;
    
    -- variables de cursor    
    c_comptador CONNCOMPTADOR.idComptador%TYPE;
    c_tipusconn CONNCOMPTADOR.TIPUSCONN%TYPE;
    c_estat CONNCOMPTADOR.ESTAT%TYPE;
     
     -- cursor que ens dir els comptadors que penjan de la c. Dist.  
       CURSOR cur_comptadors is
       SELECT idComptador,tipusconn,estat INTO c_comptador,c_tipusconn,c_estat
              FROM CONNCOMPTADOR
              WHERE p_idCDist = CONNCOMPTADOR.idCDist AND CONNCOMPTADOR.TIPUSCONN <> 'B'
              ORDER BY idComptador;
        
BEGIN
    -- si el parmetre de data baixa s nul, agafem la data baixa del sistema            
    IF p_data is null 
      THEN dataBaixaCDist := sysdate;
    ELSE dataBaixaCDist := p_data;
    END IF;
                
    -- VERIFIQUEM QUE EL PARAMETRE IDCDIST REBUT NO S NULL
    IF (p_idCDist is null) 
      THEN RAISE e_idCDist_nul;
    END IF;
       
    -- VERIFIQUEM QUE EXISTEIX A CENTRALDIST I QUE NO EST DONAT DE BAIXA
    SELECT COUNT(*) INTO numElements 
      FROM CENTRALDIST
      WHERE p_idCDist = CENTRALDIST.idCDist AND CENTRALDIST.dataBaixa is NULL;
	 	
    IF numElements <> 1
      THEN RAISE e_idCDist;
    END IF;
   
    -- VERIFIQUEM QUE dataBaixaCDist NO SIGUI MENOR QUE LA DATA DE FUNCIONAMENT
    SELECT CENTRALDIST.dataFuncionament INTO t_dataFuncionament 
      FROM CENTRALDIST 
      WHERE CENTRALDIST.idCDist = p_idCDist;
        
      t_dataFuncionament := t_dataFuncionament-1;
    
    IF greatest(t_dataFuncionament,dataBaixaCDist) = t_dataFuncionament
      THEN RAISE e_dataBaixa_erronia;
    END IF;
        
    -- VERIFIQUEM QUE NO PARTICIPA A CAP CONEXI ACTIVA (CONNCOMPTADOR)
    -- SI EST CONNECTADA A COMPTADORS, AQUESTES S'HAN DE REUBICAR.
    SELECT COUNT(*) INTO numElements 
      FROM CONNCOMPTADOR
      WHERE p_idCDist = CONNCOMPTADOR.idCDist AND CONNCOMPTADOR.tipusConn <> 'B';
        
    -- SI SUBMINISTRA A COMPTADORS, ELS LLISTEM PER QUE
    -- L'USUARI SPIGA QUINS HA DE REUBICAR.
        IF numElements > 0 THEN
          OPEN cur_comptadors;
          DBMS_OUTPUT.PUT_LINE('COMPTADORS ALS QUE ESTA CONNECTADA') ;
          DBMS_OUTPUT.PUT_LINE('IDCOMPT,TIPUSCONN,ESTAT')     ;
          DBMS_OUTPUT.PUT_LINE('------------------------')     ;
          FETCH  cur_comptadors INTO c_comptador,c_tipusconn,c_estat;
           WHILE cur_comptadors%FOUND LOOP
           DBMS_OUTPUT.PUT_LINE(c_comptador||','||c_tipusconn||','||c_estat);
           FETCH cur_comptadors into c_comptador,c_tipusconn,c_estat;
           END LOOP;
          CLOSE cur_comptadors;
        --DESPRS DE LLISTAR, LLENCEM ERROR 
        RAISE e_subministra_comptadors;
        END IF;
        
        
	-- SI NO HI HA ERRORS, DONEM DE BAIXA LA C. DIST. POSANT LA DATA BAIXA.
        UPDATE CENTRALDIST SET CENTRALDIST.dataBaixa = dataBaixaCDist WHERE CENTRALDIST.idCDist = p_idCDist;
        
        -- POSEM DE BAIXA CONNEXIONS A LES TAULES CONNCOMPTADOR I CONNLINIA
        -- ON PARTICIPA LA CENTRAL DE DISTRIBUCI DONADA DE BAIXA
        
        UPDATE CONNCOMPTADOR SET tipusConn = 'B', estat = 0 WHERE idCDist = p_idCDist;
        UPDATE CONNLINIA SET tipusConn = 'B', estat = 0 WHERE idCDist = p_idCDist;
        COMMIT;
        RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
        DBMS_OUTPUT.PUT_LINE('CENTRAL DISTRIBUCIO AMB ID '||p_idCDist||' DONADA DE BAIXA');
	
EXCEPTION

	WHEN e_idCDist_nul THEN
        	RSP := 'ERROR: PARAMETRE IDCDIST NUL';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
        WHEN e_idCDist THEN
        	RSP := 'ERROR: ID CENTRAL DIST. ' ||p_idCDist||' NO EXISTEIX O JA ESTA DONAT DE BAIXA';
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
        WHEN e_dataBaixa_erronia THEN
        	RSP := 'ERROR: DATA BAIXA HA DE SER IGUAL O MAJOR A LA DATA DE FUNCIONAMENT';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
                Rollback;

        WHEN e_subministra_Comptadors THEN
        	RSP := 'ERROR: ID CENTRAL DISTRIBUCIO ' ||p_idCDist||' SUBMINISTRA A COMPTADORS';
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR - PR_BAIXA_CENTRALDIST'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;

END;
/