CREATE OR REPLACE
PROCEDURE PR_BAIXA_CENTRALPROD (

p_idCProd IN CENTRALPROD.idCprod%TYPE,
p_data IN DATE,

RSP  OUT NOCOPY TAULALOG.RSP%TYPE 
)

/*****************************************************************************
  PROCEDIMENT:  PR_BAIXA_CENTRALPROD
    
  Dona de baixa una Central de Producci.
  
  Tant si hi ha errors com si no, s'afegeix a la 
  TAULALOG el resultat de la operaci.
    
  ENTRADA : P_IDCPROD  NUMBER(10)  , p_DATABAIXA   DATE  
  SORTIDA : RSP varchar2(300)indicant si ha anat correcte o si ha hagut errors.
  
  si parmetre DataBaixa rebut s null, s'agafa del sistema.
  
    RSP = OK
          - Es posa la data de baixa a la Central de Producci.
          - Es posa de baixa la connexi on participi la Central de Producci.
    
    RSP = ERROR + motiu, si hi ha algn error.
  
    Motius Error :
        - ERROR: ID CENTRAL PRODUCCIO p_idCProd NO EXISTEIX O JA ESTA DONAT DE BAIXA.
        - ERROR: PARAMETRE idCPROD NUL.
        - ERROR: DATA BAIXA HA DE SER IGUAL O MAJOR A LA DATA DE FUNCIONAMENT.
        - ERROR: CENTRAL PRODUCCIO p_idCProd SUBMINISTRA A CENTRALS DE DISTRIBUCIO (REUBICAR)
        
    
******************************************************************************/


IS
        -- variables errors        
        e_idCProd EXCEPTION;
	e_idCProd_nul EXCEPTION;
        e_subministra_idCDist EXCEPTION;
        e_dataBaixa_erronia EXCEPTION;
                
        -- variables insert taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_BAIXA_CENTRALPROD';
	paramEntrada taulaLog.paramEntrada%TYPE := 'p_idCProd: ' || p_idCProd || ', p_data:'|| p_data;
        paramSortida taulaLog.paramSortida%TYPE := 'RSP';
	
        
        -- data de baixa que es posar al registre.
        dataBaixaCDist  DATE;
        
        -- data d'alta recuperada de la taula
        t_dataFuncionament  DATE;
        
        -- variables d's divers. Comptador.
        numElements number := 0;
        
        -- variables del cursor 
        c_idCDist CONNLINIA.idCDist%TYPE;
        c_tipusConn CONNLINIA.tipusConn%TYPE;
        c_estat CONNLINIA.estat%TYPE;
      
      -- cursor per llistar les centrals de distribuci que pengen de la C. Prod.
       CURSOR cur_idCProd is
       SELECT idCDist,tipuscONN,estat INTO c_idCDist, c_tipusConn, c_estat
              FROM CONNLINIA
              WHERE p_idCProd = CONNLINIA.idCProd AND CONNLINIA.TIPUSCONN <> 'B'
              order by idCDist;
        
BEGIN
        -- SI EL PARMETRE DATA DE BAIXA S NULL, S'AGAFA LA DATA DEL SISTEMA          
      IF p_data is null 
        THEN dataBaixaCDist := sysdate;
      ELSE dataBaixaCDist := p_data;
      END IF;
                
      -- VERIFIQUEM QUE EL PARAMETRE idCDist REBUT NO SIGUI NULL
      IF (p_idCPROD is null) 
        THEN RAISE e_idCProd_nul;
      END IF;
       
      -- VERIFIQUEM QUE EXISTEIX A CENTRALPROD I QUE NO EST DONAT DE BAIXA
      SELECT COUNT(*) INTO numElements 
        FROM CENTRALPROD
        WHERE p_idCProd = CENTRALPROD.idCprod AND CENTRALPROD.dataBaixa is NULL;
	 	
      IF numElements <> 1
        THEN RAISE e_idCProd;
      END IF;
      
       -- VERIFIQUEM QUE dataBaixaCDist NO SIGUI MENOR QUE LA DATA DE FUNCIONAMENT
        SELECT CENTRALPROD.dataFuncionament INTO t_dataFuncionament 
          FROM CENTRALPROD
          WHERE CENTRALPROD.idCProd = p_idCProd;
        t_dataFuncionament := t_dataFuncionament-1;
        IF greatest(t_dataFuncionament,dataBaixaCDist) = t_dataFuncionament
          THEN RAISE e_dataBaixa_erronia;
        END IF;
      
      -- VERIFIQUEM QUE NO PARTICIPA A CAP CONNEXI ACTIVA (CONNLINIA)
      -- SI EST CONNECTADA A CENTRALS DISTRIBUCI, AQUESTES S'HAN DE REUBICAR.
      SELECT COUNT(*) INTO numElements 
        FROM CONNLINIA
        WHERE p_idCProd = CONNLINIA.idCProd AND CONNLINIA.tipusConn <> 'B';
        
      -- SI SUBMINISTRA A CENTRALS DE DISTRIBUCI, LES LLISTEM PER QUE
      -- L'USUARI SPIGA QUINES HA DE REUBICAR.
      IF numElements > 0 THEN
          OPEN cur_idCProd;
          DBMS_OUTPUT.PUT_LINE('CENTRALS DE DISTRIBUCIO A REUBICAR') ;
          DBMS_OUTPUT.PUT_LINE('IDCDIST,TIPUSCONN,ESTAT')     ;
          DBMS_OUTPUT.PUT_LINE('-----------------------')     ;
          FETCH  cur_idCProd INTO c_idCDist, c_tipusConn, c_estat;
           WHILE cur_idCProd%FOUND LOOP
           DBMS_OUTPUT.PUT_LINE(c_idCDist ||', '|| c_tipusConn||', '|| c_estat);
           FETCH cur_idCProd into c_idCDist, c_tipusConn, c_estat;
           END LOOP;
          CLOSE cur_idCProd;
        -- DESPRS DE LLISTAR, LLENCEM ERROR
        RAISE e_subministra_idCDist;
        END IF;
        
       
   
  ---- SI NO HI HA ERRORS, DONEM DE BAIXA LA LINIA POSANT LA DATA BAIXA.
        UPDATE CENTRALPROD SET CENTRALPROD.dataBaixa = dataBaixaCDist WHERE CENTRALPROD.idCProd = p_idCprod;
        
        --DONEM DE BAIXA LES CONNEXIONS DE LINIA A LES QUE PARTICIPI
        UPDATE CONNLINIA SET tipusConn = 'B', estat = 0 WHERE idCProd = p_idCProd;
        
        COMMIT;
      
        RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
        DBMS_OUTPUT.PUT_LINE('CENTRAL DE PRODUCCIO AMB ID '||p_idCPROD||' DONADA DE BAIXA');
	
EXCEPTION

	WHEN e_idCProd_nul THEN
        	RSP := 'ERROR: PARAMETRE idCPROD NUL';
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
        WHEN e_idCProd THEN
        	RSP := 'ERROR: ID CENTRAL PRODUCCIO ' ||p_idCProd||' NO EXISTEIX O JA ESTA DONAT DE BAIXA';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
        WHEN e_subministra_idCDist THEN
        	RSP := 'ERROR: CENTRAL PRODUCCIO ' ||p_idCProd||' SUBMINISTRA A CENTRALS DE DISTRIBUCIo (REUBICAR)';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
        
        WHEN e_dataBaixa_erronia THEN
        	RSP := 'ERROR: DATA BAIXA HA DE SER IGUAL O MAJOR A LA DATA DE FUNCIONAMENT';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
        
        WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
     
     	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR - PR_BAIXA_CENTRALPROD'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;
	
END;
/