CREATE OR REPLACE
PROCEDURE PR_BAIXA_CLIENT (

p_idClient IN CLIENT.idClient%TYPE,
p_dataBaixa IN CLIENT.dataBaixa%TYPE,

RSP OUT NOCOPY TAULALOG.RSP%TYPE
)

/*****************************************************************************
--  PROCEDIMENT:  PR_BAIXA_CLIENT
--    
--  Dona de baixa una client. 
--  Automticament es donen de baixa els contractes seus cridant a 
--      P_BAIXA_CONTRACTE per cada contracte que tingui.
--  
--  
--  Tant si hi ha errors com si no, s'afegeix a la 
--  TAULALOG el resultat de la operaci.
--    
--  ENTRADA : P_IDCLIENT  NUMBER(10)  , p_dataBaixa DATE  
--  SORTIDA : RSP VARCHAR2(300) indicant si ha anat correcte o si ha hagut errors.
--  
--  si parmetre DataBAixa rebut s null, s'agafa del sistema.
--  
--    RSP = OK
--          - Es posa la data de baixa a CLIENT. 
--          - Es posa la data de baixa als CONTRACTES que tingui..
--          
--    
--   RSP = ERROR + motiu, si hi ha algn error.
--  
--    Motius Error :
--       - ERROR: ID CLIENT p_idClient NO EXISTEIX O JA ESTA DONAT DE BAIXA.
--       - ERROR: PARAMETRE IDCLIENT NUL. 
--       - ERROR: DATA BAIXA HA DE SER IGUAL O MAJOR A LA DATA D'ALTA.
--       - Altres.
--       
******************************************************************************/

IS
    -- variables errors
    e_idClient_noexisteix EXCEPTION;
    e_idClient_nul EXCEPTION;
    e_dataBaixa_erronia EXCEPTION;

    -- variables insert taulaLOG
    nomProcediment taulaLog.procExecutat%TYPE := 'PR_BAIXA_CLIENT';
    paramEntrada taulaLog.paramEntrada%TYPE := 'P_idClient: ' || p_idClient || ',  p_dataBaixa:'|| p_dataBaixa;
    paramSortida taulaLog.paramSortida%TYPE := 'RSP';

    -- variable de la data de baixa que es posar al registre.
    dataBaixaClient  CENTRALDIST.dataFuncionament%TYPE;
    
    -- data d'alta del client
    t_dataAntiguitat  CENTRALDIST.dataFuncionament%TYPE;
    
    -- variables d's divers. Comptador.
    numElements number := 0;
    
    -- variable del cursor
    c_codiContracte CONTRACTE.codi%TYPE;
    
     
     -- cursor que ens dir els contractes actius que t el client 
       CURSOR cur_contractes is
       SELECT codi INTO c_codiContracte
              FROM CONTRACTE
              WHERE CONTRACTE.idCLIENT = p_idClient AND CONTRACTE.DATABAIXA IS  NULL;
                  
   
        
BEGIN
    -- si el parmetre de data baixa s nul, agafem la data baixa del sistema            
    IF p_dataBaixa is null 
      THEN dataBaixaClient := sysdate;
    ELSE dataBaixaClient := p_dataBaixa;
    END IF;
                
    -- VERIFIQUEM QUE EL PARAMETRE IDCLIENT REBUT NO S NULL
    IF (p_idClient is null) 
      THEN RAISE e_idClient_nul;
    END IF;
       
    -- VERIFIQUEM QUE EXISTEIX A CLIENT I QUE NO EST DONAT DE BAIXA
    SELECT COUNT(*) INTO numElements 
      FROM CLIENT
      WHERE p_idClient = CLIENT.IDCLIENT AND CLIENT.dataBaixa is NULL;
	 	
    IF numElements <> 1
      THEN RAISE e_idClient_noexisteix;
    END IF;
   
    -- VERIFIQUEM QUE dataBaixaClient NO SIGUI MENOR QUE LA DATA D'ALTA
    SELECT CLIENT.dataAntiguitat INTO t_dataAntiguitat 
      FROM CLIENT
      WHERE CLIENT.idClient = p_idClient;
        
      t_dataAntiguitat := t_dataAntiguitat-1;
    
    IF greatest(t_dataAntiguitat,dataBaixaClient) = t_dataAntiguitat
      THEN RAISE e_dataBaixa_erronia;
    END IF;
        
      
	-- SI NO HI HA ERRORS, DONEM DE BAIXA EL CLIENT POSANT LA DATA BAIXA
        UPDATE CLIENT SET CLIENT.dataBaixa = dataBaixaClient WHERE CLIENT.idClient = p_idClient;
        
        --DONEM DE BAIXA ELS CONTRACTES EN EL QUE ESTIGUI EL CLIENT
        DBMS_OUTPUT.PUT_LINE('(ES DONA DE BAIXA AUTOMATICAMENT ELS CONTRACTES');
        DBMS_OUTPUT.PUT_LINE('DEL CLIENT, PER PODER DONAR-LO DE BAIXA A ELL)');
         OPEN cur_contractes;
           FETCH  cur_contractes INTO c_codiContracte ;
           WHILE cur_contractes%FOUND LOOP
              PR_BAIXA_CONTRACTE(c_codiContracte,dataBaixaClient,RSP);
            FETCH  cur_contractes INTO c_codiContracte ;
           END LOOP;
          CLOSE cur_contractes;
        
        COMMIT;
        
        RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
        DBMS_OUTPUT.PUT_LINE('CLIENT AMB ID '||p_idClient||' DONAT DE BAIXA');

EXCEPTION

	WHEN e_idClient_nul THEN
        	RSP := 'ERROR: PARAMETRE IDCLIENT NUL';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
        WHEN e_idClient_noexisteix THEN
        	RSP := 'ERROR: ID CLIENT. ' ||p_idClient||' NO EXISTEIX O JA ESTA DONAT DE BAIXA';
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
        WHEN e_dataBaixa_erronia THEN
        	RSP := 'ERROR: DATA BAIXA HA DE SER IGUAL O MAJOR A LA DATA D''ALTA';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
                Rollback;
                  
	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR - PR_BAIXA_CLIENT '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;

END;
/