CREATE OR REPLACE
PROCEDURE PR_BAIXA_LINIA (

p_idLinia IN LINIA.idLinia%TYPE,
p_dataBaixa IN DATE,

RSP OUT NOCOPY TAULALOG.RSP%TYPE
)

/*****************************************************************************
--  PROCEDIMENT:  PR_BAIXA_LINIA
--    
--  Dona de baixa una lnia de la taula LINIA.
--  
--  Tant si hi ha errors com si no, s'afegeix a la 
--  TAULALOG el resultat de la operaci.
--    
--  ENTRADA : p_IDLINIA NUMBER(10)  , p_dataBaixa   DATE  
--  SORTIDA : RSP varchar2(300) indicant si ha anat correcte o si ha hagut errors.
--  
--  Si parmetre DataBaixa s null, s'agafa del sistema. 
--  
--    RSP = OK
--          - Es posa la data de baixa a la Linia. 
--          - Es posa de baixa la connexi CONNLINIA on participi la linia.
--    
--    RSP = ERROR + motiu, si hi ha algn error.
--  
--    Motius Error :
--        - ERROR: ID LINIA p_idLinia, NO EXISTEIX O JA ESTA DE BAIXA.
--        - ERROR: PARAMETRE idLinia NUL. 
--        - ERROR: ID LINIA ' ||p_idLinia||' SUBMINISTRA A C. DISTRIBUCIO (REUBICAR).
--        - Altres.
--        
******************************************************************************/


IS
        -- variables errors
        e_idLinia EXCEPTION;
	e_idLinia_nul EXCEPTION;
        e_subministra_idCDist EXCEPTION;
                
       -- variables insert taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_BAIXA_LINIA';
	paramEntrada taulaLog.paramEntrada%TYPE := 'P_idLinia: ' || p_idLinia || ',  p_dataBaixa:'|| p_dataBaixa;
	paramSortida taulaLog.paramSortida%TYPE := 'RSP';
	
        -- variable data de baixa que es posar al registre
        dataBaixaLinia  DATE;
        
        
        numElements number := 0;
        
        -- variables de cursor
        c_idCDist CONNLINIA.idCDist%TYPE;
        c_tipusConn CONNLINIA.tipusConn%TYPE;
        c_estat CONNLINIA.estat%TYPE;
       
       -- CURSOR QUE ENS DIR LES C. DISTRIBUCI  QUE PENGEN DE LA LINIA 
       CURSOR cur_linies is
       SELECT idCDist,tipusConn,estat INTO c_idCDist, c_tipusConn, c_estat
              FROM CONNLINIA
              WHERE p_idLinia = CONNLINIA.idLinia AND CONNLINIA.TIPUSCONN <> 'B'
              order by idCdist;
        
BEGIN
       -- SI EL PARMETRE DE DATA BAIXA S NUL, LA AGAFEM DEL SISTEMA             
      IF p_dataBaixa is null 
        THEN dataBaixaLinia := sysdate;
      ELSE dataBaixaLinia := p_dataBaixa;
      END IF;
                
      -- VERIFIQUEM QUE EL PARAMETRE idLinia REBUT NO S NULL
      IF (p_idLinia is null) 
        THEN RAISE e_idLinia_nul;
      END IF;
       
       -- VERIFIQUEM QUE EXISTEIX A LINIA I QUE NO EST DONADA DE BAIXA
      SELECT COUNT(*) INTO numElements 
        FROM LINIA
        WHERE p_idLinia = LINIA.idLinia AND LINIA.dataBaixa is NULL;
	 	
      IF numElements <> 1
        THEN RAISE e_idLinia;
      END IF;
        
      -- VERIFIQUEM QUE NO PARTICIPA A CAP CONNEXI ACTIVA (TAULA CONNLINIA)
      -- SI EST CONNECTADA A CENTRALS DISTRIBUCI, AQUESTES S'HAN DE REUBICAR.
      SELECT COUNT(*) INTO numElements 
          FROM CONNLINIA
          WHERE p_idLinia = CONNLINIA.idLinia AND CONNLINIA.tipusConn <> 'B';
        
      -- SI SUBMINISTRA A CENTRALS DE DISTRIBUCI, LES LLISTEM PER QUE
      -- L'USUARI SPIGA QUINES HA DE REUBICAR.
        IF numElements > 0 THEN
          OPEN cur_linies;
          DBMS_OUTPUT.PUT_LINE('CENTRALS DE DISTRIBUCIO A REUBICAR') ;
          DBMS_OUTPUT.PUT_LINE('IDCDIST,TIPUSCONN,ESTAT')     ;
          DBMS_OUTPUT.PUT_LINE('-----------------------')     ;
          FETCH  cur_linies INTO c_idCDist,c_tipusConn, c_estat;
           WHILE cur_linies%FOUND LOOP
           DBMS_OUTPUT.PUT_LINE(c_idCDist ||', '||c_tipusConn||', '|| c_estat);
           FETCH cur_linies into c_idCDist,c_tipusConn, c_estat;
           END LOOP;
          CLOSE cur_linies;
        -- DESPRS DE LLISTAR, LLENCEM ERROR
        RAISE e_subministra_idCDist;
        END IF;
    
        
 --------- SI NO HI HA ERRORS, DONEM DE BAIXA LA LINIA POSANT LA DATA BAIXA.
        UPDATE LINIA SET LINIA.dataBaixa = dataBaixaLinia WHERE LINIA.idLinia = p_idLinia;
        
        --DONEM DE BAIXA LES CONNEXIONS DE LINIA A LES QUE PARTICIPI
        UPDATE CONNLINIA SET tipusConn = 'B', estat = 0 WHERE idLinia = p_idLinia;
        COMMIT;
      
        RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
        
       DBMS_OUTPUT.PUT_LINE('LINIA AMB ID '||p_idLinia||' DONADA DE BAIXA');
	
EXCEPTION

        WHEN e_idLinia_nul THEN
        	RSP := 'ERROR: PARAMETRE idLinia NUL';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
        WHEN e_idLinia THEN
        	RSP := 'ERROR: ID LINIA ' ||p_idLinia||', NO EXISTEIX O JA ESTA DE BAIXA';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
        WHEN e_subministra_idCDist THEN
        	RSP := 'ERROR: ID LINIA ' ||p_idLinia||' SUBMINISTRA A C. DISTRIBUCIO (REUBICAR)';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR - PR_BAIXA_LINIA'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;
	
END;
/