CREATE OR REPLACE
PROCEDURE PR_CONSULTA_R6A(

p_localitat IN COMPTADOR.localitat%TYPE,
p_dataLectura IN LECTURES.dataLectura%TYPE,
RSP OUT NOCOPY TAULALOG.rsp%TYPE

)

/*****************************************************************************
--  PROCEDIMENT :  PR_CONSULTA_R6A
--  
--  LLISTAR ELS COMPTADORS ON EL CONSUM MENSUAL DE LA DATA PASSADA COM PARMETRE
--  HA SUPERAT EL 80% DEL CONSUM MITJ DELS COMPTADORS D'AQUELLA CIUTAT EN 
--  AQUELL PERODE DE TEMPS.
--  
--  EL LLISTAT CONSTAR DE CODI DE CONTRACTE, POTNCIA CONTRACTADA, EL PERCENTATGE
--  DE CONSUM RESPECTE AL CONSUM MITJ.
--  
--  S'HA DE TENIR EN COMPTE QUE LES LECTURES TENEN TOTES L'ULTIM DIA DEL MES.
--  
--  PRIMER DE TOT, S'HA DE CALCULAR EL 80% DEL CONSUM MITJ.
--  SELECCIONEM ELS COMPTADORS EN EL QUE EL SEU CONSUM EN AQUELLA DATA (ES
--  CONSULTA A LA TAULA LECTURES) SUPEREN AQUEST 80%.
--  AMB CADASC D'AQUEST COMPTADORS QUE COMPLEIXEN LA CONDICI, ES BUSQUEN
--  LES DADES CORRESPONENTS DE CONTRACTE.
--  
--  ENTRADA : P_LOCALITAT  VARCHAR2(20)  , p_DATACONSULTA  DATE .
--  SORTIDA : P_RSP varchar2(300) indicant si ha anat correcte o si ha hagut errors.
--  
--    P_RSP = OK. 
--    
--    P_RSP = ERROR + motiu, si hi ha algn error.
--  
--    Motius Error :
--	 - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA.
--       - Altres
--        
******************************************************************************/

IS
        -- variable error
        e_camps_nuls EXCEPTION;

        -- variables insert a taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'P_CONSULTA_R6A';
	paramEntrada taulaLog.paramEntrada%TYPE := 'P_localitat: ' || p_localitat || ',  p_dataLectura: ' || p_dataLectura;
        paramSortida taulaLog.paramSortida%TYPE := 'RSP';
	
        
       
         
        -- variable on es guarda la mitjana de les lectures 
        t_AVGLectures lectures.consumLectura%TYPE :=0;
        
        -- variable on es guarda el 80% del consum que s'ha de superar
         vuitantaPercent lectures.consumLectura%type := 0;
        
        -- variable on es guarden les dades pel llistat
        t_codicontracte CONTRACTE.codi%TYPE;
        t_KWcontractats CONTRACTE.KWContractats%TYPE;
        
        -- variables del cursor
        c_consumLectura LECTURES.consumLectura%TYPE;
        c_tantpercent lectures.consumLectura%type;
        c_idComptador lectures.idComptador%TYPE;
        
        numElements number;
      
        

        -- DEFINIM EL CURSOR ON ES GUARDARA ELS RESULTATS DE LA CONSULTA
        -- seleccionem el consum, el percentatge respecte de la t_AVGLectures, i el idComptador.
        CURSOR c_resultat_R6A is SELECT lectures.CONSUMLECTURA, lectures.CONSUMLECTURA*100/t_AVGLectures , LECTURES.IDCOMPTADOR  
            FROM LECTURES WHERE LECTURES.CONSUMLECTURA > vuitantaPerCent AND LECTURES.DATALECTURA = p_dataLectura AND LECTURES.IDCOMPTADOR IN 
            (      SELECT COMPTADOR.IDCOMPTADOR
                FROM COMPTADOR
                WHERE COMPTADOR.LOCALITAT = p_localitat) 
            ORDER BY LECTURES.CONSUMLECTURA ;


  BEGIN
        
        -- VERIFIQUEM QUE NO HI HAGI NULS ALS PARAMETRES ENTRADA
        IF ((p_localitat is null)  or (p_dataLectura is null)) THEN RAISE e_camps_nuls;
        END IF;
       
        
      
      
         -- CALCULEM EL 80% DEL CONSUM MITJ DELS COMPTADORS EN LA DATA I CIUTAT DONADES COM A PARAMETRES.
        -- LA QUANTITAT ES GUARDA A LA VARIABLE VUITATNTAPERCENT
	SELECT AVG(LECTURES.CONSUMLECTURA) INTO t_AVGLectures FROM LECTURES WHERE
        LECTURES.DATALECTURA = p_dataLectura AND LECTURES.IDCOMPTADOR IN (
        SELECT COMPTADOR.IDCOMPTADOR
        FROM COMPTADOR
        WHERE COMPTADOR.LOCALITAT = p_localitat);
        
        IF t_AVGLectures IS NULL THEN t_AVGLectures :=0; END IF;
        vuitantaperCent := t_AVGLectures * 0.8;
    
	DBMS_OUTPUT.PUT_LINE('RESULTAT DE LA CONSULTA R6A PER   LOCALITAT: '||P_LOCALITAT||'    DATA:'||p_dataLectura);
	DBMS_OUTPUT.PUT_LINE('MITJANA DE CONSUM : '|| t_AVGLectures ||'KWh.      80% = '|| vuitantaperCent||' KWh.');

        -- OBRIM EL CURSOR ON ESTAN ELS COMPTADORS QUE HAN TINGUT CONSUMS EN AQUELLA DATA I CIUTAT SUPERIOR
        -- AL 80% DEL CONSUM MIG.

        OPEN c_resultat_R6A;
        FETCH c_resultat_R6A INTO c_consumLectura, c_tantpercent, c_idComptador;
        IF c_resultat_R6A%NOTFOUND THEN
          DBMS_OUTPUT.PUT_LINE('NO HI HA REGISTRES A LLISTAR');
        ELSE
        DBMS_OUTPUT.PUT_LINE('COMPTADORS QUE SUPEREN AQUESTS '||vuitantapercent||' KWh DE CONSUM ')     ;
        WHILE c_resultat_R6A%FOUND LOOP
            -- MITJANANT EL IDCOMPTADOR, SELECCIONEM EL CONTRACTE I KW CONTRACTATS DE LA TAULA CONTRACTE.
           SELECT CONTRACTE.codi , CONTRACTE.KWContractats into t_codicontracte, t_KWcontractats
           FROM CONTRACTE
           WHERE CONTRACTE.idComptador = c_idComptador;

           DBMS_OUTPUT.PUT_LINE('CONTRACTE: '|| t_codicontracte || '. KWCONTRACT: '||t_KWcontractats
             ||' KW.  %CONSUM RESPECTE MITJANA ' ||c_tantpercent||'%. ('||c_consumLectura||' Kwh)');
          
        FETCH c_resultat_R6A INTO c_consumLectura, c_tantpercent, c_idComptador;
        END LOOP;
        CLOSE c_resultat_R6A;
        END IF;

        RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
	
        COMMIT;

  EXCEPTION

	WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
    
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR - PR_CONSULTA_R6A '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		ROLLBACK;

END;
/