CREATE OR REPLACE
PROCEDURE PR_CONSULTA_R6B (
p_num IN NUMBER,
RSP OUT NOCOPY TAULALOG.RSP%TYPE
)
/*****************************************************************************
--  PROCEDIMENT :  PR_CONSULTA_R6B
--  
--  Llistar les 10 centraletes de distribuci que distribueixen ms
--  energia anualment, mostrant la seva adrea, energia emesa en l'any (tenint 
--  en compte que es basa en les lectures, i ser l'any de la ltima lectura)
--  i la  mxima energia que permet subministrar en un any (camp MAXKwh).
--  
--  
--  ENTRADA : P_PNUM  NUMBER. 
--  SORTIDA : RSP varchar2(300)  indicant si ha anat correcte o si ha hagut errors.
--  
--  P_num ser el nombre de Centraletes que volem llistar. En aquest cas 10, per
--  es pot variar segons si ens interessa saber en un futur les 20 centraletes
--  de ms distribuci, per exemple
--  
--    RSP = OK. 
--    
--    RSP = ERROR + motiu, si hi ha algn error.
--  
--    Motius Error :
--    - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA.
--    - Altres.
******************************************************************************/
	
IS
	
            
        -- variables d'errors
        e_camps_nuls EXCEPTION;
        
        -- variables de insert a taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_CONSULTA_R6B';
	paramEntrada taulaLog.paramEntrada%TYPE := 'p_num (numPosicions Llistat):'|| p_num;
	paramSortida taulaLog.paramSortida%TYPE := 'RSP';
	
        -- variable d's divers.
        numElements number;
        
        -- comptador del cursor que son les posicions del llistat
        numPosicions NUMBER := p_num;
     
        -- variables llegides de la taula CENTRALDIST
        t_adreca CENTRALDIST.ADRECA%TYPE;
        t_codiPostal CENTRALDIST.codiPostal%TYPE;
        t_localitat CENTRALDIST.LOCALITAT%TYPE;
        t_pais CENTRALDIST.pais%TYPE;
        t_maxKWh CENTRALDIST.maxKWh%TYPE;
               
        -- variables de cursor
        c_energia CENTRALDIST.maxKWh%type;
        c_cDist CENTRALDIST.idCDist%type;
        
        -- DEFINIM EL CURSOR ON ES GUARDARA ELS RESULTATS DE LA CONSULTA
        -- sumem els KWh subministrats  per C. Dist. que es pot mirar a
        -- la taula CONNCOMPTADOR (camp KWhConnexio).
        -- Recordar que aquest camp s'actualitza amb cada consum de lectura
        --  que es va afegint, i que cada any es reinicia.
        -- En aquest camp KWhConnexio estan els KWh subministrats anualment
        -- per de cada parell C.Distribucio - Comptador.
        
        -- s'inclouen tots excepte els que estan de baixa
        CURSOR cur_resultat_R6B is
        SELECT SUM(CONNCOMPTADOR.KWhConnexio), CONNCOMPTADOR.idCDist 
            FROM CONNCOMPTADOR 
            WHERE CONNCOMPTADOR.tipusConn <> 'B'
        group by CONNCOMPTADOR.idCDist
        order by SUM(CONNCOMPTADOR.KWhConnexio)desc;

	
  BEGIN
        --VERIFIQUEM QUE NO HI HAGI CAMPS NULS ALS PARAMETRES      
        IF (p_num is null) THEN RAISE e_camps_nuls;
        END IF;
        
        DBMS_OUTPUT.PUT_LINE('LLISTAT DE LES '||P_NUM||' C. DIST QUE DISTRIBUEIXEN MES ENERGIA EN L''ANY');
        DBMS_OUTPUT.PUT_LINE('---------------------------------------------------------------------------------------');
       
       
       -- SEGONS EL NUMERO DONAT COM A PARAMETRE, LLISTAREM TANTES POSICIONS
        OPEN cur_resultat_R6B;
        FETCH cur_resultat_R6B INTO c_energia, c_cDist;
            IF cur_resultat_R6B%NOTFOUND 
              THEN DBMS_OUTPUT.PUT_LINE('NO HI HA REGISTRES A LLISTAR');
            ELSE 
              --DBMS_OUTPUT.PUT_LINE('ENERGIA (KWH)        IDCDIST');
              --DBMS_OUTPUT.PUT_LINE('--------------      ----------')     ;
            WHILE cur_resultat_R6B%FOUND AND numPosicions <> 0 
              LOOP
              -- seleccionem l'adrea de la C.Dist. pel llistat
              SELECT adreca, codiPostal, localitat, pais, maxKWh
                  INTO t_adreca, t_codiPostal, t_localitat, t_pais, t_maxKWh
                  FROM CENTRALDIST
                  WHERE idcDist = c_cDist;
              
              -- llistem el registre del cursor tractat amb la informaci
              DBMS_OUTPUT.PUT_LINE('CDist: '||c_cDist||'.   '||t_adreca||' CP:'|| t_codiPostal ||'.  '|| t_localitat
              ||'.  '||t_pais||'.  ENERGIA EMESA :' ||c_energia||'kWh. '||'  MAXKWH ->'||t_maxKWh||' KWh')     ;
           
              numPosicions := numPosicions-1 ;
           
            FETCH cur_resultat_R6B INTO c_energia,c_cDist;
              END LOOP;
        CLOSE cur_resultat_R6B;
            END IF;
	
        SELECT COUNT(CONNCOMPTADOR.IDCDIST) INTO numElements
            FROM CONNCOMPTADOR 
            WHERE conncomptador.estat = 1 And CONNCOMPTADOR.tipusConn <> 'B';
        IF (  numElements < p_num)
            THEN DBMS_OUTPUT.PUT_LINE('NO HI HA '||P_NUM||' CENTRALETES');
          END IF;
            
            
        RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
	COMMIT;
	
EXCEPTION

	WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
               
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		DBMS_OUTPUT.PUT_LINE('ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA');
        	Rollback;
                
       
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		DBMS_OUTPUT.PUT_LINE('ERROR : ERROR SENSE TRACTAR -  PR_CONSULTA_R6B '||SQLCODE||'-'||SQLERRM);
		DBMS_OUTPUT.PUT_LINE('Rollback');
 		ROLLBACK;
	
			

	END;
/