CREATE OR REPLACE
PROCEDURE PR_CONSULTA_R6C (

p_num IN NUMBER,
p_anyo IN CHAR,
RSP  OUT NOCOPY  TAULALOG.RSP%TYPE 
)

/*****************************************************************************
--  PROCEDIMENT :  PR_CONSULTA_R6C
--  
--  Consulta que llistar les 10 lnies de comunicaci ms carregades en relaci
--  a la seva capacitat mxima. 
--  Mostrar el idLinia, la crrega (que ser subministre anual de la lnia /
--  maxKWh anual) i la energia a la que es pot ampliar segons la Central de 
--  producci connectada.
--  
--  
--  ENTRADA : P_PNUM   NUMBER  , p_ANYO  CHAR(4) 
--  
--  SORTIDA: RSP varchar2(300) indicant si ha anat correcte o errors.
--  
--  P_num ser el nombre de lnies que volem llistar. Si volem 10 posicions, el 
--  parmetre ser 10, per es pot variar. 
--  
--  p_anyo ser l'any en el que volem fer la consulta, si ho deixem
--   com a null, tindr en compte el de la ltima lectura feta (que s lo que 
--   demanara el client). Si posem un any, ho far amb aquell any.
--   
--    RSP = OK. 
--    
--    RSP = ERROR + motiu, si hi ha algn error.
--  
--    Motius Error :
--	 - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA.
--       - Altres.
--         
******************************************************************************/


	
IS
	
        -- variable error
        e_camps_nuls EXCEPTION;
        
        -- variable insert taulaLOG        
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_CONSULTA_R6C';
	paramEntrada taulaLog.paramEntrada%TYPE := 'P_num (numPosicions Llistat):'|| p_num||',  p_anyo '|| p_anyo;
        paramSortida taulaLog.paramSortida%TYPE := 'RSP';
	
        -- variables per saber l'any amb qu es fa el llistat
        anyomax char(4);
        t_MAXdataLectura date;

        -- variables d's divers.
        numElements number;
        
        
        -- variable amb la posici a tractar pel cursor.
        numPosicions NUMBER := p_num;
        
        -- variable per calcular ampliacio que pot tenir la linia
        ampliacio LINIA.MAXKWH%TYPE := 0;
        idCProdLinia CONNLINIA.IDCPROD%TYPE;
        MAXkwhCProd LINIA.MAXKWH%TYPE;
        sumaKWhLinies LINIA.MAXKWH%TYPE;
 	
        -- variables de cursor
        c_consum LINIA.MAXKWH%TYPE;
        c_idlinia LINIA.IDLINIA%TYPE;
        c_carrega NUMBER(4,2);
        
        -- DEFINIM EL CURSOR ON ES GUARDARA ELS RESULTATS DE LA CONSULTA
        -- sumem els KWh subministrats  per la linia mitjanant les lectures
        -- guardem la linia i a cada grup de linia, guardem la crrega
        -- de la linia (suma / maxkwh que pot suportar)
        
        -- la forma d'arribar des de LECTURES fins LINIA
        -- s a traves de les claus LECTURES - CONNCOMPTADOR - CONNLINIA - LINIA
        -- tenint en compte noms les connexions actives.
            
       CURSOR cur_resultat_R6C is
       SELECT SUM(LECTURES.consumLectura)                  
              ,connlinia.idLINIA 
                ,SUM(LECTURES.consumLectura)*100/(SELECT maxkwh FROM LINIA WHERE idlinia = connlinia.idlinia)
            INTO c_consum, c_idlinia, c_carrega
        FROM LECTURES, connlinia,conncomptador
        WHERE lectures.idcomptador = conncomptador.idcomptador 
              AND  to_char(lectures.datalectura,'yyyy') = to_char(anyomax)
              AND conncomptador.idCdist = connlinia.idCDist
              AND connlinia.estat = 1 and conncomptador.estat =1
        GROUP BY connlinia.idLINIA
        ORDER BY SUM(LECTURES.consumLectura)*100/(SELECT maxkwh FROM linia WHERE idlinia = connlinia.idlinia) DESC; 

	
BEGIN
        --VERIFIQUEM QUE NO HI HAGI CAMPS NULS ALS PARAMETRES      
        IF (p_num is null) THEN RAISE e_camps_nuls;
        END IF;
        
        -- SI PARMETRE P_ANYO ES NULL, LLAVORS CALCULEM EL LLISTAT 
        -- AMB LA DATA ANY DE LA ULTIMA LECTURA EFECTUADA,
        -- SI NO, FAREM EL LLISTAT AMB L'ANY INDICAT
        
        IF p_anyo is null
          THEN
            select max(datalectura) INTO t_MAXdataLectura
            FROM LECTURES;
            ANYOMAX := TO_CHAR(t_MAXdataLectura,'yyyy');
          ELSE
            ANYOMAX := p_anyo;
        end if;
        DBMS_OUTPUT.PUT_LINE('LLISTAT DE LES '||p_num||' LINES MES CARREGADES A L''ANY '|| anyomax);
        DBMS_OUTPUT.PUT_LINE('--------------------------------------------------------------------------');
       
       -- SEGONS EL NUMERO DONAT COM A PARAMETRE, LLISTAREM TANTES POSICIONS
        OPEN cur_resultat_R6C;
        FETCH cur_resultat_R6C INTO c_consum, c_idlinia, c_carrega;
       
            IF cur_resultat_R6C%NOTFOUND 
              THEN DBMS_OUTPUT.PUT_LINE('NO HI HA REGISTRES A LLISTAR');
            ELSE 
            WHILE cur_resultat_R6C%FOUND AND numPosicions <> 0 
              LOOP
              
              -- mirem el mxim de pot produr la C. Prod de que penja per poder
              -- saber la ampliaci que pot tenir.
             SELECT DISTINCT CENTRALPROD.MAXKWH, CENTRALPROD.IDCPROD INTO MAXkwhCProd, idCProdLinia
                  FROM CENTRALPROD INNER JOIN CONNLINIA
                 ON CENTRALPROD.IDCPROD = CONNLINIA.IDCPROD 
                 AND CONNLINIA.IDLINIA = c_idLinia AND CONNLINIA.ESTAT = 1;
             
             -- despres sumem els mxims de les linies (incloent la tractada)
             -- a les que subministra la c. producci
             
             SELECT SUM(LINIA.MAXKWH) INTO sumaKWhLinies FROM LINIA
              WHERE LINIA.IDLINIA IN (SELECT CONNLINIA.IDLINIA
              FROM CONNLINIA
               WHERE CONNLINIA.IDCPROD = idCProdLinia);
             
             -- i la resta ser a lo que es pot ampliar
             ampliacio := maxKWhCProd - sumaKWhLinies;
             
              
              
              DBMS_OUTPUT.PUT_LINE('IDLINIA '||c_idlinia||' -  CONSUM :'||c_consum||' KWh - CARREGA : '||c_carrega|| '%. AMPLIACIO: '||ampliacio||' KWh')     ;
              numPosicions := numPosicions-1 ;
            FETCH cur_resultat_R6C INTO c_consum,c_idlinia,c_carrega;
              END LOOP;
            CLOSE cur_resultat_R6C;
            END IF;
	
        SELECT COUNT(CONNLINIA.IDLINIA) INTO numElements
            FROM CONNLINIA 
            WHERE connlinia.estat = 1 And CONNlinia.tipusConn <> 'B';
        IF (  numElements < p_num)
            THEN DBMS_OUTPUT.PUT_LINE('NO HI HA '||P_NUM||' LINIES');
          END IF;
            
            
        RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
	COMMIT;
	
EXCEPTION

	WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
             	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
       
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR - PR_CONSULTA_R6C '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;

END;
/