CREATE OR REPLACE
PROCEDURE PR_CONSULTA_R6D 

(
RSP OUT NOCOPY taulaLog.RSP%TYPE
)


/*****************************************************************************
-- PROCEDIMENT :  PR_CONSULTA_R6D
--    
--  LLISTAT DELS CLIENTS QUE TENEN COMPTADORS EN ALTA DISPONIBILITAT.
--  
--  ELS COMPTADORS HAN DE PENJAR DE MS D'UNA C. DISTRIBUCI I AQUESTES 
-- DE CENTRALS DE PRODUCCI DIFERENTS.
--  
--  ENTRADA : CAP PARMETRE.
--  SORTIDA : RSP indicant si ha anat correcte o si ha hagut errors.
--            
--            Llistat amb el DNI - ID del client, el codi contracte i el 
--              model del comptador dels clients que tinguin comptadors 
--	      en alta disponibilitat.
--  
--    RSP = OK. 
--    
--    RSP = ERROR + motiu, si hi ha algun error.
--  
--    Motius Error :
--          - ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA
--	  - Altres.
******************************************************************************/
IS
        -- variable error
        e_camps_nuls EXCEPTION;
        
        -- variables insert taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_CONSULTA_R6D';
	paramEntrada taulaLog.paramEntrada%TYPE := 'NO HI HA PARAMETRES D''ENTRADA' ;
        paramSortida taulaLog.paramSortida%TYPE := 'RSP';
	
        
             
        numElements NUMBER(10);
        numComptadors NUMBER(10);
        numCProds NUMBER(10);
                
	 -- variables dels camps fabricant i modelcompt dels select
        t_fabricant MODELCOMPT.fabricant%TYPE;
        t_modelCompt MODELCOMPT.modelCompt%TYPE;
        
        t_codiContracte CONTRACTE.codi%TYPE;
        t_nif CLIENT.NIF%TYPE;
        t_idClient CLIENT.idClient%TYPE;
	
        
        -- variables cursor
        c_idComptador CONTRACTE.idComptador%TYPE;
        
	
        
        -- DEFINIM EL CURSOR DELS COMPTADORS QUE ESTAN  EN ALTA DISPONIBILITAT I 
        -- MIREM QUE EL CURSOR PENGI DE MS D'UNA CENTRAL DE DISTRIBUCI
        -- I I TAMB DE MS D'UNA CENTRAL DE PRODUCCI
       
        CURSOR cur_resultat_R6D is 
         
            SELECT conncomptador.idcomptador
            FROM conncomptador, connlinia, comptador
            WHERE conncomptador.idcdist = connlinia.idcdist  AND CONNLINIA.TIPUSCONN<>'B' 
             AND comptador.databaixa is null
            AND conncomptador.idcomptador IN 
                  -- SI EL COMPTADOR APAREIX MS D'UNA VEGADA A LA TAULA
                  -- CONNCOMPTADOR EN CONNEXIONS QUE NO ESTAN DE BAIXA
                  -- SIGNIFICAR QUE EL COMPTADOR PENJA DE MS D'UNA C. DIST.
                     (SELECT IDCOMPTADOR
                    FROM CONNCOMPTADOR
                    WHERE CONNCOMPTADOR.TIPUSCONN<>'B'
                    GROUP BY IDCOMPTADOR
                    HAVING COUNT(*) >1)
          GROUP BY conncomptador.idcomptador
            -- ES SELECCIONARAN ELS COMPTADORS QUE PENGIN DE MS D'UNA C. PROD. DIFERENT.
          HAVING COUNT(distinct(connlinia.idcprod))>1
          
  	;

     
   
BEGIN
               
   	
        OPEN cur_resultat_R6D;
        FETCH cur_resultat_R6D INTO c_idComptador ;
        IF cur_resultat_R6D%NOTFOUND THEN
          DBMS_OUTPUT.PUT_LINE('NO HI HA REGISTRES A LLISTAR');
        ELSE 
            DBMS_OUTPUT.PUT_LINE('CLIENTS AMB COMPTADORS EN ALTA DISPONIBILITAT' );
            DBMS_OUTPUT.PUT_LINE('---------------------------------------------' );
             DBMS_OUTPUT.PUT_LINE('IDCLIENT, NIF,  CONTRACTE, MODEL');
              DBMS_OUTPUT.PUT_LINE('');
        WHILE cur_resultat_R6D%FOUND LOOP
                  SELECT contracte.codi, CONTRACTE.idclient, CLIENT.NIF INTO t_codiContracte, t_idClient,t_nif
                         
                     FROM CONTRACTE, CLIENT 
                   WHERE CONTRACTE.dataBaixa IS NULL
                    AND CLIENT.DATABAIXA IS NULL
                     AND CONTRACTE.IDCOMPTADOR = c_idcomptador
                  
                  AND CONTRACTE.IDCLIENT IN (SELECT IDCLIENT
                        FROM CLIENT
                        WHERE CLIENT.databaixa is null)
              
                         
              AND CONTRACTE.IDCLIENT = CLIENT.IDCLIENT  ; 
        
            -- AMB IDCOMPTADOR DEL CONTRACTE, ACONSEGUIM EL MODEL DEL COMPTADOR
            
            SELECT MODELCOMPT.fabricant, MODELCOMPT.modelCompt INTO t_fabricant, t_modelCompt
              FROM MODELCOMPT
              WHERE MODELCOMPT.IDMODEL IN (select COMPTADOR.idmodel FROM COMPTADOR WHERE c_idComptador = COMPTADOR.idcomptador);
         
         DBMS_OUTPUT.PUT_LINE(t_idClient||' , '||t_nif ||' , '|| t_codiContracte|| ' , '||t_modelCompt);
          
          FETCH cur_resultat_R6D INTO c_idComptador;
          END LOOP;
          CLOSE cur_resultat_R6D;
        END IF;
	
        
        COMMIT;
        RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
      
	
EXCEPTION

	WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
        	Rollback;
                
       
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR - PR_CONSULTA_R6D '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
 		ROLLBACK;

END;
/