CREATE OR REPLACE
PROCEDURE PR_CONSULTA_R6E (

p_idCprod IN CENTRALPROD.idCprod%TYPE,
p_dataInici IN LECTURES.TIPUSLECTURA%TYPE,
p_dataFi IN LECTURES.TIPUSLECTURA%TYPE,
RSP OUT NOCOPY TAULALOG.RSP%TYPE
	
)
/*****************************************************************************
--  PROCEDIMENT :  PR_CONSULTA_R6E
--    
--  DONADA UNA CENTRAL PRODUCCI I INTERVAL DE TEMPS, SABER EL CONSUM DELS
--  COMPTADORS QUE DEPENEN D'AQUESTA CENTRAL I ENERGIA PRODUDA.
--
--  SUPOSEM QUE TANT EL CONSUM CON LA PRODUCCI S LA MATEIXA QUANTITAT.
--  
--  MITJANANT LES TAULES CONNLINIA I CONNCOMPTADOR PODEM SABER QUINS COMPTADORS
-- DEPENEN D'UNA CENTRAL DE PRODUCCI I MIRAR LES SEVES LECTURES EN EL PERODE
--  DONAT.
--  
--  ES LLISTA EL RESULTAT.
--  
--  ENTRADA : P_IDCPROD NUMBER(10) , p_ DATAINICI   DATE , p_DATAFI  DATE.
--  SORTIDA : RSP varchar2(300) indicant si ha anat correcte o amb errors.
--    RSP = OK. 
--    
--    RSP = ERROR + motiu, si hi ha algnn error.
--
--   Motius error:
--      - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA.
--      - ERROR: PARAMETRES DATA INICI > DATA FINAL.
--      - ERROR: CENTRAL DE PRODUCCIO ID p_idCprod NO EXISTEIX.
--      - Altres.
--  
******************************************************************************/

IS
	-- variables errors
        e_camps_nuls EXCEPTION;
        e_dates_erronies EXCEPTION;
        e_idcProd_noexisteix EXCEPTION;

        -- variables insert taulaLOG
	nomProcediment taulaLog.procExecutat%TYPE := 'PR_CONSULTA_R6E';
	paramEntrada taulaLog.paramEntrada%TYPE := 'P_idCProd: '||p_idCProd||', p_dataInici: '||p_dataInici||', p_dataFi: '||p_dataFi ;
	paramSortida taulaLog.paramSortida%TYPE := 'RSP';

        -- variable on es guarda la suma dels consums de la idCPROD al select
        t_sumaConsum LECTURES.consumLectura%TYPE;
        
        
        numElements NUMBER;
	
BEGIN
       -- verifiquem que els parmetres d'entrada no siguin nuls.        
       IF ((p_idCProd is null)  or (p_dataInici is null)or (p_dataFi is null)) THEN RAISE e_camps_nuls;
        END IF;
        
        -- verifiquem que la data inici sigui ms gran a la del final
      IF p_dataInici > p_dataFi THEN RAISE e_dates_erronies;
        END IF; 
        
        
       DBMS_OUTPUT.PUT_LINE('CENTRAL DE PRODUCCIO:' ||p_idCprod );
       DBMS_OUTPUT.PUT_LINE('INTERVAL DE TEMPS : '||p_dataInici||' - '||p_dataFi );
      
      
      -- VERIFIQUEM QUE EXISTEIXI LA CENTRAL DE PRODUCCIO
      SELECT COUNT(*) into numElements
        FROM CENTRALPROD
        WHERE CENTRALPROD.IDCPROD = p_idCProd;
      
      IF numElements = 0 
        THEN RAISE e_idCprod_noexisteix;
      END IF;
      
      -- sumem els consums en el perode indicat dels comptadors que
      -- depenen de la central de producci donada.
      SELECT SUM(LECTURES.consumLectura) INTO t_sumaConsum 
        FROM LECTURES 
        WHERE LECTURES.dataLectura BETWEEN p_dataInici AND p_dataFi
        AND   LECTURES.idComptador IN (SELECT conncomptador.idcomptador
                          FROM conncomptador, connlinia
                          WHERE connlinia.idcProd = p_idCProd 
                              AND conncomptador.idcdist = connlinia.idcdist 
                              AND conncomptador.estat = 1 AND connlinia.estat =1);


        IF t_sumaConsum is null THEN  t_sumaConsum := 0; END IF;
        
          DBMS_OUTPUT.PUT_LINE('(SUPOSEM QUE CONSUM I ENERGIA PRODUiDA ES LA MATEIXA QUANTITAT)');
         DBMS_OUTPUT.PUT_LINE('ENERGIA PRODUIDA :' ||t_sumaConsum||' KWh');
          
       	
        RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
	
        COMMIT;
	
EXCEPTION

	WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
	  	Rollback;
                
       WHEN e_dates_erronies THEN
        	RSP := 'ERROR: PARAMETRES DATA INICI > DATA FINAL';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
        WHEN e_idCProd_noexisteix THEN
        	RSP := 'ERROR: CENTRAL DE PRODUCCIO ID '||p_idCprod||' NO EXISTEIX';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR - PR_CONSULTA_R6E '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;
	
END;
/