CREATE OR REPLACE
PROCEDURE PR_CONSULTA_R6F (

p_dataInici IN LECTURES.TIPUSLECTURA%TYPE,
p_dataFi IN LECTURES.TIPUSLECTURA%TYPE,
RSP OUT NOCOPY taulaLog.RSP%TYPE 

)

/*****************************************************************************
--  PROCEDIMENT :  PR_CONSULTA_R6F
--    
--  DONAT UN PERIODE DE TEMPS, PERCENTATGE DE LECTURES TELEMATIQUES I PRESENCIALS
-- 
--  PASOS A REALITZAR:
--      - SELECCIONAR LES LECTURES DEL PERIODE DE TEMPS I VEURE QUANTES HI HA
--      - COMPTAR LES QUE SON TELEMATIQUES I PRESENCIALS I CALCULAR EL PERCENTATGE
--  
--  ENTRADA : P_DATAINICI  DATE , p_DATAFI DATE.
--  SORTIDA : RSP VARCHAR2(300) indicant si ha anat correcte o amb errors.
--  
--  MOSTRAR EN LLISTAT EL PERCENTATGE DE LECTURES DE CADA TIPUS EN AQUELL PERIODE.  
--  
--    RSP = OK. 
--    
--    RSP = ERROR + motiu, si hi ha algun error.
--  
--   Motius Error :
--	 - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA.
--       - ERROR: PARAMETRES DATA INICI > DATA FINAL.
--       - Altres.
--
******************************************************************************/
	
IS
	
        -- variables errors
        e_camps_nuls EXCEPTION;
        e_dates_erronies EXCEPTION;
        
        -- variables del insert taulaLOG        
	nomProcediment taulaLog.procExecutat%TYPE := 'PR_CONSULTA_R6F';
	paramEntrada taulaLog.paramEntrada%TYPE := 'P_dataInici: '||p_dataInici||', p_dataFi: '||p_dataFi ;
        paramSortida taulaLog.paramSortida%TYPE := 'RSP';
	
         -- variables on es guarden els resultats dels selects
        numLectures NUMBER (15);
        numLecturesTelematiques NUMBER(15);
        numLecturesPresencials NUMBER(15);
        percentT NUMBER(4,1);
        percentP NUMBER(4,1);
     	
BEGIN
       -- verifiquem que els parmetres no siguin nuls        
       IF ((p_dataInici is null)or (p_dataFi is null)) THEN RAISE e_camps_nuls;
        END IF;

      -- verifiquem que la data inici no sigui ms gran que la data final.
      IF p_dataInici > p_dataFi THEN RAISE e_dates_erronies;
        END IF; 
        
      
      -- Fem les consultes dels resultats  
      SELECT COUNT(*) INTO numLectures FROM LECTURES
          WHERE LECTURES.dataLectura BETWEEN p_dataInici and p_dataFi;
          
      SELECT COUNT(*) INTO numLecturesTelematiques FROM LECTURES
          WHERE (LECTURES.dataLectura BETWEEN p_dataInici and p_dataFi) and LECTURES.tipusLectura = 'T';
          
      SELECT COUNT(*) INTO numLecturesPresencials FROM LECTURES
          WHERE (LECTURES.dataLectura BETWEEN p_dataInici and p_dataFi) and LECTURES.tipusLectura = 'P';
        
       percentT := numLecturesTelematiques * 100 /numLectures;
       percentP := numLecturesPresencials * 100 / numLectures;
      
      --TREIEM EL RESULTAT PER PANTALLA   
      DBMS_OUTPUT.PUT_LINE('INTERVAL DE TEMPS :'||p_dataInici||'-'||p_dataFi );
      DBMS_OUTPUT.PUT_LINE('QUANTITAT DE LECTURES:' ||numLectures
            || '. TELEMATIQUES '|| numLecturesTelematiques||' ('||percentT||'%)'
              ||'. PRESENCIALS '||numLecturesPresencials||' ('||percentP||'%)');
         
       RSP := 'OK';
       PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
   COMMIT;
	
EXCEPTION

	WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
        	Rollback;
                
       WHEN e_dates_erronies THEN
        	RSP := 'ERROR: PARAMETRES DATA INICI > DATA FINAL';
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
        	Rollback;

	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR - PR_CONSULTA_R6F'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
 		ROLLBACK;
END;
/