CREATE OR REPLACE
PROCEDURE PR_CONSULTA_R6G (

p_anysAntiguitat IN COMPTADOR.anyFabricacio%TYPE,
RSP OUT NOCOPY taulaLog.RSP%TYPE 
)


/*****************************************************************************
-- PROCEDIMENT :  PR_CONSULTA_R6G
--    
--  DONAT UN NOMBRE D'ANY, LLISTAR ELS COMPTADORS QUE TINGUIN AQUELLA ANTIGUITAT.
--    
--  PASOS A REALITZAR:
--      - EXTREURE L'ANY DEL SISTEMA
--      - SELECCIONAR AQUELLS COMPTADORS EN QUE 
--		ANY SISTEMA - ANY DE FABRICACI = ANY PARMETRE
--  
--  ENTRADA : ANY
--  SORTIDA : RSP indicant si ha anat correcte o si ha hagut errors.
--            MOSTRAR ELS COMPTADORS AMB ELS ANYS D'ANTIGUITAT DONAT.
--  
--    RSP = OK. 
--    
--    RSP = ERROR + motiu, si hi ha algn error.
--  
--    Motius Error :
--	 - ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA.
--       - ERROR: ANY INCORRECTE  (s negatiu o > 9999)
--       - Altres.
--
******************************************************************************/
	
IS
        -- variables de errors
        e_camps_nuls EXCEPTION;
        e_any_erroni EXCEPTION;	
       
        -- variables insert taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_CONSULTA_R6G';
        paramEntrada taulaLog.paramEntrada%TYPE := ' P_anysAntiguitat: '|| p_anysAntiguitat ;
        paramSortida taulaLog.paramSortida%TYPE := 'RSP';
	
       
        -- variable on es guardar l'any del sistema per calcular.       
        anySistema COMPTADOR.anyFabricacio%TYPE;

        -- variable cursor amb els comptadors que compleixen la condici.
        c_idComptador COMPTADOR.idComptador%TYPE;
      
       -- DEFINIM EL CURSOR AMB ELS COMPTADORS QUE COMPLEIXEN ELS ANYS 
       -- D'ANTIGUITAT DONATSI QUE NO ESTIGUIN DE BAIXA
      
      CURSOR cur_consulta_R6G IS 
        SELECT COMPTADOR.idComptador  
        FROM COMPTADOR
        WHERE anySistema-COMPTADOR.anyFabricacio = p_anysAntiguitat
        AND COMPTADOR.DATABAIXA IS NULL;
        
BEGIN
        -- verifiquem que el parmetre no s nul
        IF (p_anysAntiguitat is null) THEN RAISE e_camps_nuls;
        END IF;
       
       -- verifiquem que el parmetre s un any vlid.
       if p_anysAntiguitat < 0 or p_anysAntiguitat >9999 then RAISE e_any_erroni;
       end if;
       
      -- guardem a anySistema l'any del sistema.       
      select TO_NUMBER(TO_CHAR(sysdate, 'yyyy')) into anySistema from dual;
        
      
      -- encapalament del llistat
      DBMS_OUTPUT.PUT_LINE('ANY ANTIGUITAT DONAT :'||p_anysAntiguitat ); 
      DBMS_OUTPUT.PUT_LINE('ANY SISTEMA: '||anySistema ); 
      DBMS_OUTPUT.PUT_LINE('LLISTAT DELS COMPTADORS FABRICATS A L''ANY: ' || to_char(anySistema-p_anysAntiguitat) ); 
     
        OPEN cur_consulta_R6G;
        FETCH cur_consulta_R6G INTO c_idComptador ;
        IF cur_consulta_R6G%NOTFOUND THEN 
            DBMS_OUTPUT.PUT_LINE('NO HI HA COMPTADORS AMB '||p_anysAntiguitat||' D''ANTIGUITAT');
        ELSE
        WHILE cur_consulta_R6G%FOUND LOOP
        DBMS_OUTPUT.PUT_LINE('IDCOMPTADOR :'||c_idComptador);
        
        FETCH cur_consulta_R6G into c_idComptador;
        END LOOP;
        CLOSE cur_consulta_R6G;
       END IF; 
       	
        RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
	
EXCEPTION
                
      WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
                rollback;
        	
	WHEN e_any_ERRONI THEN
        	RSP := 'ERROR: ANY INCORRECTE (HA DE TENIR 4 XIFRES';
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
                rollback;
        	
        WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR - PR_CONSULTA_R6G '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		rollback;

END;
/