create or replace FUNCTION F_VERIFICAR_MAXIMSKWH (

p_dataLectura IN LECTURES.datalectura%TYPE,
p_consumLectura in LECTURES.consumLectura%TYPE,
p_idComptador IN LECTURES.idComptador%TYPE,
p_idCDist IN CENTRALDIST.idCDist%type,
p_idLinia in CONNLINIA.idLinia%TYPE,
p_idCProd in CENTRALPROD.idcProd%TYPE,
RSP OUT NOCOPY taulalog.rsp%type

)
RETURN  BOOLEAN

/*****************************************************************************
  FUNCIO:  F_VERIFICAR_MAXIMSKWH
    
  Amb cada lectura, es verificar que no es sobrepassi els mxims anuals de kwh de
  la central de distribuci, lnia i central de producci de la que rep energia
  el comptador al que estem anotant la lectura.
  
  Grcies a les taules CONNCOMPTADOR i CONNLINIS es pot fer un mapa de la
  xarxa i saber a quins comptadors subministra la C. Prod, etc. etc.
  
  Aquesta funci es crida desde P_ALTA_LECTURES, abans d'insertar-la.
  
  S'afegeix a la TAULALOG el resultat de la operaci.
    
  ENTRADA : p_DATALECTURA DATE , p_CONSUMLECTURA NUMBER(8,1),  
          p_IDCOMPTADOR  NUMBER(10), p_IDCDIST NUMBER(10), p_IDLINIA NUMBER(10), 
          p_IDCPROD  NUMBER(10) . 
          
  SORTIDA: RSP varchar2(300)  indicant si ha anat correcte o si ha hagut errors.
  
  RETORNA : TRUE si es sobrepassa el mxim.
            FALSE si no es sobrepassa.
    
    RSP = OK : NO ES SOBREPASSA ELS MXIMS.
    
    RSP = ERROR + motiu, si hi ha algn error.
  
    Missatges Error :
  	- ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA NO SUPORTATS.
        - ES SOBREPASSARIA EL MAXKWH ANUAL QUE SUPORTA LA C. DIST p_idCDist
        - ES SOBREPASSARIA EL MAXKWH ANUAL QUE SUPORTA LA LINIA p_idlinia
        - ES SOBREPASSARIA EL MAXKWH ANUAL QUE SUPORTA LA C. PROD p_idCProd.
        
        

******************************************************************************/

IS

 	e_camps_nuls EXCEPTION;
        e_maxKWh_idCDist exception;
        e_maxKWh_idLinia exception;
        e_maxKWh_idCProd exception;
        
        nomProcediment taulaLog.procExecutat%TYPE := 'F_VERIFICAR_MAXIMSKWH';
	paramEntrada taulaLog.paramEntrada%TYPE := 'P_dataLectura: ' || p_dataLectura || ', p_idComptador: ' 
          || p_idComptador || ', p_consumLectura :'|| p_consumLectura;

	paramSortida taulaLog.paramSortida%TYPE := 'RSP';
	
        
        numElements number := 0;
        sumKWh CENTRALDIST.maxKWh%TYPE;
        maxKWh CENTRALDIST.maxKWh%TYPE;
        retorn boolean;
BEGIN
 retorn := false;
 RSP := 'OK' ; 
    -- VERIFIQUEM QUE ELS PARMETRES NO SIGUIN NULS     
    IF ((p_dataLectura is null) or (p_idComptador is null)  or (p_consumLectura is null))
      THEN retorn := true;
      RAISE e_camps_nuls;
    END IF;
     
  -- VERIFICACI DE MAXIM DE CENTRAL DE DISTRIBUCI
  -- sumem les lectures del mateix any que la lectura que estem verificant
  -- que rebin energia de la mateixa central de distribuci
  SELECT SUM(LECTURES.consumLectura) into sumKWH
      FROM LECTURES
      WHERE TO_CHAR(lectures.datalectura,'yyyy') = TO_CHAR(p_dataLectura,'yyyy')
      AND LECTURES.idComptador IN
          (SELECT COMPTADOR.idComptador
            FROM COMPTADOR
            WHERE COMPTADOR.idComptador IN(
                SELECT CONNCOMPTADOR.idComptador
                FROM CONNCOMPTADOR
                WHERE conncomptador.estat = 1 and CONNCOMPTADOR.idCDist = p_idCdist ));
              
  -- I ho comparem amb el  mxim suportat
  SELECT CENTRALDIST.MAXKWH INTO maxkwh
    FROM CENTRALDIST    
    WHERE CENTRALDIST.IDCDIST= p_idCDist;
        
  IF sumKwh IS NULL THEN SUMkwH:=0; END IF;
                  
  -- AFEGIM EL CONSUM ACTUAL ALS KWH SUBMINISTRATS PER LA C DIST 
  -- PER VEURE SI ES SOBREPASSARIA EL SEU MXIM         
  sumKWh := sumKWh + p_consumLectura;
-- DBMS_OUTPUT.PUT_LINE('SUMA KWH C.DIST :' ||sumkwh||', MAXIM SUPORTAT C. DIST '||p_idCDist||' : '|| maxkwh);
  IF sumkwh > maxkwh
    THEN 
   -- SI SOBREPASSA, RETORN := TRUE I LLENCEN ERROR
    retorn := true;
    
    RAISE e_maxKWh_idCDist;
  END IF;

--------------------------------------------------------------------------
  -- VERIFICACI DE MAXIM DE LINIA
  -- sumem les lectures del mateix any que la lectura que estem verificant
  -- que rebin energia de la mateixa linia     
  SELECT SUM(LECTURES.consumLectura) into sumKWH
    FROM LECTURES
    WHERE TO_CHAR(lectures.datalectura,'yyyy') = TO_CHAR(p_dataLectura,'yyyy')
        AND LECTURES.idComptador IN
            (SELECT COMPTADOR.idComptador
              FROM COMPTADOR
              WHERE COMPTADOR.idComptador IN(
                  SELECT CONNCOMPTADOR.idComptador
                  FROM CONNCOMPTADOR
                  WHERE conncomptador.estat = 1 and CONNCOMPTADOR.idCDist IN
                     (SELECT CONNLINIA.IDCDIST 
                      FROM CONNLINIA
                      WHERE CONNLINIA.estat = 1 and CONNLINIA.IDlinia = p_idLinia)));
              
  -- I ho comparem amb el  mxim suportat
  SELECT LINIA.MAXKWH INTO maxkwh
    FROM LINIA
    WHERE LINIA.IDLINIA= p_idLinia;
        
  IF sumKwh IS NULL THEN SUMkwH:=0; END IF;
                  
  -- AFEGIM EL CONSUM ACTUAL ALS KWH SUBMINISTRATS PER LA LINIA 
  -- PER VEURE SI ES SOBREPASSARIA EL SEU MXIM         
  sumKWh := sumKWh + p_consumLectura;
  --DBMS_OUTPUT.PUT_LINE('SUMA KWH LINIA :' ||sumkwh||', MAXIM SUPORTAT LINIA '||p_idLinia||' : '|| maxkwh);
  IF sumkwh > maxkwh
    THEN 
    -- SI SOBREPASSA, RETORN := TRUE I LLENCEN ERROR
    retorn := true;
    RAISE e_maxKWh_idlinia;
  END IF;

-------------------------------------------------------------

  -- VERIFICACI DE MAXIM DE CENTRAL DE PRODUCCI
  -- sumem les lectures del mateix any que la lectura que estem verificant
  -- que rebin energia de la mateixa central de Producci
  SELECT SUM(LECTURES.consumLectura) into sumKWH
    FROM LECTURES
    WHERE TO_CHAR(lectures.datalectura,'yyyy') = TO_CHAR(p_dataLectura,'yyyy')
          AND LECTURES.idComptador IN
            (SELECT COMPTADOR.idComptador
              FROM COMPTADOR
              WHERE COMPTADOR.idComptador IN(
                  SELECT CONNCOMPTADOR.idComptador
                  FROM CONNCOMPTADOR
                  WHERE conncomptador.estat = 1 and CONNCOMPTADOR.idCDist IN
                        (SELECT CONNLINIA.IDCDIST 
                          FROM CONNLINIA
                          WHERE CONNLINIA.estat = 1 and CONNLINIA.idCProd = p_idCProd)));
              
   -- I ho comparem amb el  mxim suportat
  SELECT CENTRALPROD.MAXKWH INTO maxkwh
      FROM CENTRALPROD
  WHERE CENTRALPROD.IDCPROD = p_idCProd;
        
  IF sumKwh IS NULL THEN SUMkwH:=0; END IF;
                  
  -- AFEGIM EL CONSUM ACTUAL ALS KWH SUBMINISTRATS PER LA C. PRODUCCIOI 
  -- PER VEURE SI ES SOBREPASSARIA EL SEU MXIM         
  sumKWh := sumKWh + p_consumLectura;
  --DBMS_OUTPUT.PUT_LINE('SUMA KWH C PROD :' ||sumkwh||', MAXIM SUPORTAT C PROD '||p_idCProd||' : '|| maxkwh);
  IF sumkwh > maxkwh
  -- SI SOBREPASSA, RETORN := TRUE I LLENCEN ERROR
    THEN  retorn := true;
    RAISE e_maxKWh_idCPROD;
  END IF;

  RETURN retorn;
        

EXCEPTION

	WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA NO SUPORTATS';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		DBMS_OUTPUT.PUT_LINE(RSP);
                return retorn;

	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		DBMS_OUTPUT.PUT_LINE(RSP);
                return retorn;
   
	 WHEN e_maxKWh_idCDist THEN
        	RSP := 'ES SOBREPASSARIA EL MAXKWH ANUAL QUE SUPORTA LA C. DIST '||p_idCDist ;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		DBMS_OUTPUT.PUT_LINE(RSP);
                return retorn;

        
         WHEN e_maxKWh_idLinia THEN
        	RSP := 'ES SOBREPASSARIA EL MAXKWH ANUAL QUE SUPORTA LA LINIA '||p_idlinia ;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		DBMS_OUTPUT.PUT_LINE(RSP);
        	return retorn;
                
        WHEN e_maxKWh_idCprod THEN
        	RSP := 'ES SOBREPASSARIA EL MAXKWH ANUAL QUE SUPORTA LA C. PROD '||p_idCProd ;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		DBMS_OUTPUT.PUT_LINE(RSP);
        	return retorn;

	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		DBMS_OUTPUT.PUT_LINE(RSP);
		return retorn;

END;
/