create or replace PROCEDURE PR_ACTUALITZAR_HISTORICCPROD (

p_dataLectura IN LECTURES.datalectura%TYPE,
p_idComptador IN LECTURES.idComptador%TYPE,
p_consumLectura IN LECTURES.consumLectura%TYPE,

RSP OUT NOCOPY TAULALOG.RSP%TYPE
)

/*****************************************************************************
--  PROCEDIMENT:  PR_ACTUALITZAR_HISTORICCPROD
--    
--  Amb cada lectura, actualitzar l'histric de la central de producci.
--  Si per aquella data  i central de producci hi ha entrada a la taula, el consum
--  s'afegeix.
--  Si no hi ha cap entrada a la taula amb la data i central, s'inserta.
--  
--  (S'ha de tenir en compte que les lectures tenen sempre com a data lectura 
--  l'ultim dia del mes)
--  
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la 
--  TAULALOG el resultat de la operaci.
--    
--  ENTRADA : P_DATALECTURA DATE, p_IDCOMPTADOR NUMBER(10), p_CONSUMLECTURA NUMBER(8,1) . 
--  SORTIDA : RSP (varchar2) indicant si ha anat correcte o si ha hagut errors.
--  
--    
--    RSP = OK : S'inserta registre.
--    
--    RSP = ERROR + motiu, si hi ha algn error.
-- 
--    Motius Error :
--  	- ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA NO SUPORTATS.
--        - Altres.
--
******************************************************************************/


IS
      
  -- Variables de cursor
  c_idcomptador COMPTADOR.idcomptador%TYPE;
  c_idCDist CONNLINIA.idCDist%TYPE;
  c_idCProd CONNLINIA.idCProd%TYPE;
  c_idLinia CONNLINIA.idLinia%TYPE;

  -- variables procedents de selects
  t_dataLectura LECTURES.datalectura%TYPE;
  t_consumLectura LECTURES.consumLectura%TYPE;

  -- Variables d'error
  e_camps_nuls EXCEPTION;

  	
  -- variables temporals 
  numElements number := 0;

  -- variables pel taulaLOG
  nomProcediment taulaLog.procExecutat%TYPE := 'PR_ACTUALITZAR_HISTORICCPROD';
  paramEntrada taulaLog.paramEntrada%TYPE := 'P_dataLectura: ' || p_dataLectura || ', p_idComptador: ' 
          || p_idComptador || ', p_consumLectura :'|| p_consumLectura;
  paramSortida taulaLog.paramSortida%TYPE := 'RSP';

 -- Cursor que ens dir el recorregut CProd - Linia - CDist - Comptador  
  -- Recordem que el comptador noms rep subministrament d'una CDist (de la   
  -- connexi activa i aquesta d'una linia i CProd.

  CURSOR cur_linia IS 
  SELECT connlinia.idcprod, connlinia.idLinia, conncomptador.idcdist, conncomptador.idcomptador
    FROM conncomptador, connlinia
    WHERE conncomptador.idcdist = connlinia.idcdist and p_idComptador = conncomptador.idComptador 
     AND conncomptador.estat = 1 and connlinia.estat =1
     AND CONNCOMPTADOR.TIPUSCONN <> 'B' AND CONNLINIA.TIPUSCONN <> 'B';


BEGIN
  -- verifiquem que els parmetres no siguin null 	
    IF ((p_dataLectura is null) or (p_idComptador is null)  or (p_consumLectura is null))
          THEN RAISE e_camps_nuls;
        END IF;
       OPEN cur_linia;

  --En teoria, noms es far un FETCH ja que cada comptador noms tindr un
  -- un cam actiu fins la Central de Producci
  -- (S'implementa aix per si en un futur es canvia el sistema i pot
  --  rebre energia de dos o ms centrals a la vegada)
  FETCH cur_linia into c_idCProd, c_idLinia, c_idCDist, c_idComptador;
      WHILE cur_linia%FOUND LOOP
       -- DBMS_OUTPUT.PUT_LINE(T_IDcOMPTADOR||','||c_idCDist||','||c_idLinia||','||c_idCProd);   
       IF p_CONSUMLECTURA > 0 THEN    
           SELECT COUNT(*) INTO numElements
           FROM HISTORICCPROD
           WHERE HISTORICCPROD.IDCPROD = c_idCProd
           AND HISTORICCPROD.DATAPROD = p_datalectura;
           
           -- si hi ha entrada, sumem consum a aquella data i CProd
           IF numElements <> 0   THEN
             UPDATE HISTORICCPROD
            SET HISTORICCPROD.KWHPRODUITS = HISTORICCPROD.KWHPRODUITS + p_consumLectura
             WHERE HISTORICCPROD.IDCPROD = c_idCProd
            AND HISTORICCPROD.DATAPROD =p_datalectura;
           ELSE 
           
            --Si no hi ha entrada amb aquella data i Cprod, insertem a la taula
            INSERT INTO HISTORICCPROD VALUES ( c_idCProd, p_datalectura,p_consumLectura);
           end if;
          COMMIT;
          END IF;
          fetch cur_linia into c_idCProd, c_idLinia, c_idCDist,c_idComptador;
          END LOOP;
          CLOSE cur_linia;
  
  RSP := 'OK';
  PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);  
  --DBMS_OUTPUT.PUT_LINE('HISTORICCPROD ACTUALITZAT');
	
EXCEPTION

	WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA NO SUPORTATS';
               
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		DBMS_OUTPUT.PUT_LINE(RSP);
    
	
        WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG(nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		DBMS_OUTPUT.PUT_LINE(RSP);
                
  	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		DBMS_OUTPUT.PUT_LINE(RSP);

	END;
/