create or replace PROCEDURE PR_ACTUALITZAR_ME1 (

p_idCProd  IN ME1.idCProd%TYPE,
p_consum IN ME1.consum%TYPE,
RSP OUT NOCOPY taulalog.RSP%TYPE

)

--/*****************************************************************************
--  PROCEDIMENT:  PR_ACTUALITZAR_ME1
--  
--  AQUEST PROCEDIMENT S'EXECUTARA AL INSERTAR UNA LECTURA PER RECALCULAR I
--  TENIR ACTUALITZADA LA TAULA ME1.
--  
--  Es crida des de PR_ALTA_LECTURA.
--  S'actualitza el registre amb clau idCProd passada com parmetre 
--  i es suma el consum de la lectura. 
--  Ho faig per totals absoluts, sense separar per anys.
--
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la 
--  TAULALOG el resultat de la operaci.
--    
--  ENTRADA : P_IDCPROD  NUMBER(10) ,  p_CONSUM  NUMBER(12,1) .
--
--  SORTIDA : RSP varchar2(300) indicant si ha anat correcte o amb  errors.
--         
--   RSP = OK : S'inserta registre a la taula CPCARBO.
--    
--   RSP = ERROR + motiu, si hi ha algUn error.
--  
--    Motius Error :
--      - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA.
--      - Altres.
--    
--******************************************************************************/


IS
	-- variable error
        e_camps_nuls EXCEPTION;
       
        -- variables insert taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_ACTUALITZAR_ME1';
	paramEntrada taulaLog.paramEntrada%TYPE := 'P_idCProd : ' || p_idCProd || ', p_consum: ' || p_consum;
	paramSortida taulaLog.paramSortida%TYPE := 'RSP';
      
        -- variables temporals d's divers
        numElements number;
BEGIN
                
      -- VERIFIQUEM ELS NULS DELS PARAMETRES D'ENTRADA
      IF   (p_idcprod is null) or (p_consum is null)
          THEN RAISE e_camps_nuls;
      END IF;
    
    -- MIREM SI HI HA ANOTACIONS A LA TAULA ME1 PER AQUEST C. PROD
    SELECT COUNT(*) INTO numElements
        FROM ME1
        WHERE ME1.idCProd = p_idCProd;
        
    -- SI JA HI HA UNA ENTRADA AMB AQUESTA IDCPROD, SUMEM EL CONSUM
    -- SI S LA PRIMERA, LA INSERTEM I POSEM EL CONSUM.
      
    IF numElements = 1
        THEN UPDATE ME1 SET ME1.consum = ME1.consum + p_consum WHERE ME1.idCProd = p_idCProd;
    ELSE
        INSERT INTO ME1 VALUES (p_idCProd, p_consum);
    END IF;
    
    COMMIT;
    RSP := 'OK';
    PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
         
    
      
EXCEPTION

	WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                  
               
	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
      
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR A PR_ACTUALITZAR_ME1 - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		rOLLBACK;
	
END;
/