create or replace PROCEDURE PR_ACTUALITZAR_ME2 (

p_idLinia  IN ME2.idLinia%TYPE,
p_dataLectura IN date,
p_idcomptador IN LECTURES.idCOMPTADOR%TYPE,
p_consumLectura IN LECTURES.consumLectura%TYPE,

RSP OUT NOCOPY taulalog.RSP%TYPE

)

--/*****************************************************************************
--  PROCEDIMENT:  PR_ACTUALITZAR_ME2
--  
--  QUAN S'INSERTA UNA LECTURA, ES CRIDA A AQUEST MODUL ESTADISTIC
--  Calcular, segons la lnia i any de la lectura, la energia mitja dels
--  comptadors que reben energia d'aquella linia en aquell any
--  Insertar a la taula amb clau idLinia-any el consum mig i el numero 
--      de comptadors (aquest camp servir per agilitzar els  clculs  en aquest
--      procedure i a P_ACTUALITZAR_M4)
--  
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la 
--  TAULALOG el resultat de la operaci.
--    
--  ENTRADA : P_IDLINIA  NUMBER(10), p_DATALECTURA DATE, 
--          p_IDCOMPTADOR  NUMBER(10), p_CONSUMLECTURA NUMBER(12,1).
--
--  SORTIDA : RSP VARCHAR2(300) indicant si ha anat correcte o si ha hagut errors.
--          
--   RSP = OK : S'inserta registre a la taula ME2.
--    
--   RSP = ERROR + motiu, si hi ha algn error.
--  
--    Motius Error :
--      - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA.
--      - Altres.
--
--******************************************************************************/


IS
	-- variable errors
        e_camps_nuls EXCEPTION;
       
        -- variables insert taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_ACTUALITZAR_ME2';
	paramEntrada taulaLog.paramEntrada%TYPE := 'p_idLinia: ' || p_idLinia || ', p_dataLectura :'||p_dataLectura || ', p_idComptador: '||p_idComptador||', p_consumLectura :'||p_consumLectura;
	paramSortida taulaLog.paramSortida%TYPE := 'RSP';
      
        -- variables de SELECTS de taules
        t_numComptadors number;
        t_energiaMitja ME2.energiaMitja%TYPE;
        
        -- variables de clculs varis i comptadors
        numElements number :=0;
        energia ME2.energiaMitja%TYPE;
        anyo_lectura ME2.anyo%type;
BEGIN
      -- guardem a la variable l'any de la lectura
      anyo_lectura := to_number(to_char(p_dataLectura,'YYYY'));
      
      
      -- VERIFIQUEM ELS NULS DELS PARAMETRES D'ENTRADA
      IF   (p_idlinia is null) or (p_dataLectura is null) or (p_idcomptador is null)
          THEN RAISE e_camps_nuls;
      END IF;
    
    -- PRIMER, COMPTEM ELS COMPTADORS, INCLOENT ELS DONATS DE BAIXA DURANT L'ANY
    -- JA QUE ELS CONSUMS D'AQUESTS TAMB S'HAN DE TENIR EN COMPTE.
    -- LES CONNEXIONS HAN DE SER LES PRINCIPALS
    SELECT COUNT(comptador.idComptador) INTO t_numComptadors
      FROM CONNLINIA INNER JOIN CONNCOMPTADOR INNER JOIN COMPTADOR 
      ON COMPTADOR.IDCOMPTADOR = CONNCOMPTADOR.IDCOMPTADOR 
        AND  CONNCOMPTADOR.TIPUSCONN = 'P'
              AND COMPTADOR.DATABAIXA  IS NULL  
                   OR  to_date(to_char(COMPTADOR.dataBaixa,'YYYY')) = to_date(to_char(p_dataLectura,'YYYY'))     
      ON CONNCOMPTADOR.IDCDIST = CONNLINIA.IDCDIST AND CONNLINIA.IDLINIA = p_idLinia AND CONNLINIA.TIPUSCONN = 'P'
       ;
  
    -- PRIMER MIREM SI HI HA ENTRADA PER AQUELLA LINIA I ANY
    -- JA QUE EN AQUEST CAS, AMB ELS CAMPS ENERGIA MITJA I NUM COMPTADORS
    -- PODEM ESBRINAR EL CONSUM QUE TENIEM FINS ARA
    -- I AIX ESTALVIA FER LA SUMA AMB LES LECTURES D'AQUELL ANY I IDLINIA. (MS COSTS EN TEMPS)
    
    SELECT COUNT (*) INTO numElements FROM ME2 
    WHERE IDLINIA = P_IDLINIA 
    AND ANYO = anyo_lectura;
    
   -- SI HI HA REGISTRE IDLINIA-ANYO, AGAFEM ENERGIA MITJA I NUM COMPTADORS 
   -- PER SABER EL CONSUM TOTAL DE LA LINIA EN AQUELL ANY ABANS DE LA LECTURA 
    IF  numElements > 0  
    THEN 
        SELECT ME2.energiaMitja, ME2.numComptadors into t_energiaMitja,t_numComptadors
          FROM ME2 
          WHERE IDLINIA =  P_IDLINIA AND ANYO = anyo_lectura;
     
     -- ENERGIA CONSUMIDA ABANS DE LA LECTURA SER LA VARIABLE energia
        
        energia := t_energiaMitja * t_numComptadors;  
        
     -- CALCULEM NOVA ENERGIA MITJA AMB AQUESTES DADES, SUMEN EL CONSUM DE LA LECTURA
     -- I DIVIDIM PER NOMBRE DE COMPTADORS
        
        t_energiaMitja := (energia + p_consumlectura)/t_numComptadors;
    
      -- UPDATEM EL MODUL ESTADISTIC AMB LA MITJANA DE CONSUMS DELS COMPTADORS
      -- D'AQUELLA LINIA I ANY
        UPDATE ME2 
          SET ME2.energiaMitja = t_energiaMitja
          WHERE IDLINIA = p_idLinia AND me2.ANYO = anyo_lectura;
          --DBMS_OUTPUT.PUT_LINE('ENERGIA MITJA DESPRES LECTURA :'|| t_energiaMitja|| ' sumatori ;'||t_energiaMitja*t_numComptadors);   

    --SI ES LA PRIMERA ENTRADA A LA TAULA D'AQUESTA LINIA, INSERTEM ENERGIA MITJA 
    ELSE
      energia := p_consumLectura;
      t_energiaMitja := p_consumlectura/t_numComptadors;
    
      INSERT INTO ME2 VALUES (p_idLinia,anyo_lectura,t_energiaMitja, t_numComptadors); 
    END IF;
    
    COMMIT;
    RSP := 'OK';
    PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
  
    
  
EXCEPTION

	WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
               
	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
      
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR A PR_ACTUALITZAR_ME2 - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;
	
END;
/