CREATE OR REPLACE
PROCEDURE PR_ACTUALITZAR_ME3 (

p_idLinia  IN ME3.idLinia%TYPE,
RSP OUT NOCOPY taulalog.RSP%TYPE

)

--/*****************************************************************************
--  PROCEDIMENT:  PR_ACTUALITZAR_ME3
--  
--  Procediment que actualitza la taula ME3 posant la lnia que ha tingut ms
--  CRREGA en valors absoluts (KWh absoluts d'energia consumida pels comptadors 
--  que pengen d'ella, des de la seva alta)
--
--  Desprs d'insertar una nova lectura amb el seu consum, des de
--  PR_ALTA_LECTURA es crida a aquest procediment.
--  
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la 
--  TAULALOG el resultat de la operaci.
--    
--  ENTRADA : P_IDLINIA  NUMBER(10).
--
--  SORTIDA : RSP varchar2 indicant si ha anat correcte o si ha hagut errors.
--         
--   RSP = OK . S'actualitza la taula ME3.
--    
--    RSP = ERROR + motiu, si hi ha algn error.
--  
--    Motius Error :
--      - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA.
--      - Altres.
--  
--******************************************************************************/


IS
	-- variables errors
        e_camps_nuls EXCEPTION;
       
        -- variables insert taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_ACTUALITZAR_ME3';
	paramEntrada taulaLog.paramEntrada%TYPE := 'P_idLinia: ' || p_idLinia;
	paramSortida taulaLog.paramSortida%TYPE := 'RSP';
      
        -- variables de camps agafats de taules
        t_consumMax  ME3.CONSUM%type :=0;
        t_idLiniaMax ME3.idLinia%TYPE :=0;
      
        -- variables temporals i d's divers        
        numElements number;
        
        -- variable de la suma del consum de la Linia de la lectura.
        v_consumIdLinia lectures.consumLectura%TYPE;
   
        
BEGIN
                
   
  -- SUMAREM ELS CONSUMS DELS COMPTADORS DE LA LINIA A LA QUE PERTANY LA NOVA LECTURA
  -- (ES FA AMB ELS CONSUMS TOTALS)
  -- LES CONNEXIONS HAN DE SER LES ACTIVES
    
    SELECT SUM(LECTURES.consumLectura)                  
          INTO v_consumIdLinia
          FROM LECTURES, connlinia,conncomptador
          WHERE lectures.idcomptador = conncomptador.idcomptador 
                AND  conncomptador.idCdist = connlinia.idCDist
                AND connlinia.estat = 1 and conncomptador.estat =1
                AND connlinia.idLinia = p_idLinia ;
    
    -- MIREM SI HI HA REGISTRES A LA TAULA ME3 I SI HI HA, AGAFEM ELS VALORS
    SELECT COUNT(*)  INTO numElements FROM ME3;
    IF numElements > 0 THEN
        SELECT consum, idlinia INTO t_consumMax, t_idLiniaMax
        FROM ME3
        WHERE posRanking =1;
     
           -- SI EL CONSUM DE LA LINIA QUE ESTEM TRACTANT s MAJOR A LA QUE HI HAVIA
           -- A LA TAULA ME3, ACTUALITZEM.
          IF v_consumidLinia > t_consumMax THEN
              UPDATE ME3 SET posRanking =1,
                       idLinia = p_idLinia,
                       consum = v_consumidLinia;
          END IF;
     
     -- SI NO HI HA VALORS A LA TAULA ME3, INSERTEM (PRIMER CONSUM LLEGIT)
     ELSE
     INSERT INTO ME3 VALUES (1,p_idlinia,v_consumIdLinia);
     END IF;
  
    COMMIT;
    RSP := 'OK';
    PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
         
  
           
EXCEPTION

	WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
		PR_INSERT_LOG(nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                  
               
	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
      
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR A PR_ACTUALITZAR_ME3 - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;
	
END;
/