CREATE OR REPLACE
PROCEDURE PR_ACTUALITZAR_ME5 (

RSP OUT NOCOPY taulalog.RSP%TYPE
)

--/*****************************************************************************
--  PROCEDIMENT:  PR_ACTUALITZAR_ME5

--  AQUEST PROCEDIMENT ES CRIDA AL INSERTAR UNA NOVA LECTURA DES DE P_ALTA_LECTURA
--  
--  CALCULA EL NUMERO DE C. DE PRODUCCIO QUE NO ARRIBEN AL 30% DEL SEU MXIM DE PRODUCCI.
--  SEPARAT PER ANYS. I ACTUALITZA LA TAULA ME5.

--  S'UTILITZA ELS VALORS DE LA TAULA HISTORICCPROD EN EL QUE ESTAN ELS CONSUMS
--  PER MESOS DE CADA CENTRAL DE PRODUCCI I QUE S'ACTUALITZA AMB CADA LECTURA 
-- (EN AQUEST TFC TENIM EL COMPTE QUE LO PRODUIT S IGUAL A LO QUE S'HA CONSUMIT)
--  
--  AQUESTS REGISTRES DE LA TAULA HISTORICCPROD S'AGRUPARAN PER ANY 
--  I PER CADA ANY I C. PRODUCCI ES COMPARAR ELS KWH PRODUITS AMB EL MXIM 
--  QUE SUPORTA AQUESTA PER VEURE SI GENERA MENYS DEL 30%.
--  
--  A LA TAULA ME5 ES POSAR PER CADA ANY QUE HI HAGI CONSUM, EL NOMBRE DE
--  CENTRALS DE PRODUCCI QUE NO ARRIBEN A AQUEST 30%.
--
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la 
--  TAULALOG el resultat de la operaci.
--    
--   ENTRADA : NO HI HA CAP PARMETRE D'ENTRADA.
--              
--   SORTIDA : RSP indicant si ha anat correcte o si ha hagut errors.
--          
--   RSP = OK : S'inserta registre a la taula ME6.
--   RSP = ERROR + motiu, si hi ha algn error.
--
--   Motius Error :
--      - Altres.
--      
--
--******************************************************************************/


IS
	-- variables errors
        e_camps_nuls EXCEPTION;
       
        -- variables insert taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_ACTUALITZAR_ME5';
	paramEntrada taulaLog.paramEntrada%TYPE := 'NO REP PARMETRES' ;
	paramSortida taulaLog.paramSortida%TYPE := 'RSP';
      
        -- variables d's divers i comptadors
        numElements number :=0;
        numCProd number :=0;
        cont number := 0;
        
        -- variable per si volem modificar el percentatge que ens diu l'enunciat
        percentatge number:= 30;
        
        -- variable per saber si ha canviat d'any el registre del cursor
        anyo_temp ME5.ANYO%type;

        -- variable de mxim agafat de select de taula
        t_maxKWhCProd CENTRALPROD.MAXKWH%type;        
        
        -- variables de cursor
        c_anyo ME5.anyo%TYPE;
        c_idCProd HISTORICCPROD.IDCPROD%TYPE;
        c_KWhProduits HISTORICCPROD.KWHPRODUITS%TYPE;
       
        -- CURSOR ON ES GUARDARAN A CADA REGISTRE LA SUMA DE CADA ANY DE KWH DE CADA IDCPROD
        -- ORDENAT PER  ANY I CENTRAL DE PRODUCCIO.
        CURSOR c_resultat_ME5 is
        SELECT SUM(KWHPRODUITS), TO_CHAR(DATAPROD,'YYYY'), IDCPROD
                FROM HISTORICCPROD
                GROUP BY TO_CHAR(DATAPROD,'YYYY'), IDCPROD
                ORDER BY   TO_CHAR(DATAPROD,'YYYY'),IDCPROD;
            
      
BEGIN
    
  OPEN c_resultat_ME5;
  FETCH c_resultat_ME5 INTO c_KWhProduits,c_anyo, c_idCprod ;
    IF c_resultat_ME5%NOTFOUND 
    THEN DBMS_OUTPUT.PUT_LINE('NO HI HA REGISTRES A HISTORICCPROD D''ON AGAFAR LES DADES');
    ELSE 
        WHILE  c_resultat_ME5%FOUND  loop
        -- SEGONS LA C. PROD LLEGIDA DEL CURSOR, SELECCIONEM EL MAXIM DE KWH QUE POT PRODUIR LA C. PROD
        -- PER VEURE SI EL CONSUM LLEGIT D'AQUELL ANY NO ARRIBA AL SEU 30%
        -- SI NO ARRIBA, SUMEM 1 A NUMELEMENTS QUE AL CANVIAR D'ANY SER LA QUANTITAT A INSERTAR A LA TAULA ME5
                
            SELECT CENTRALPROD.MAXKWH INTO t_maxKWhCProd FROM CENTRALPROD WHERE CENTRALPROD.idCprod= c_idCProd;  
            IF c_KWhProduits < (t_maxKWhCProd * percentatge)/100 THEN numElements := numElements +1; END IF;
            -- DBMS_OUTPUT.PUT_LINE('idcprod :' || c_idCprod||'. MAXKWH: '||t_maxKWhCProd||' .KWhProduits :'||c_kwhproduits);
            
            anyo_temp := c_anyo;  
                
          FETCH c_resultat_ME5 INTO c_KWhProduits,c_anyo, c_idCprod ;
             
           -- SI CANVIA D'ANY AL CURSOR, O NOMS HI HAVIA UN REGISTRE AL CURSOR, 
           -- LLAVORS UPDATEM A LA TAULA M5 L'ANY TRACTAT (O INSERTEM SI S NOU)
           -- I POSEM EL NOMBRE DE C. PRODUCCIO QUE NO ARRIBEN AL PERCENTATGE DE PRODUCCIÓ
                 
          IF anyo_temp <> c_anyo OR c_resultat_me5%notfound  THEN
          SELECT COUNT(*) INTO cont FROM ME5 WHERE anyo_temp = ME5.anyo;
          
              IF CONT = 0 THEN
                  INSERT INTO ME5 VALUES (anyo_temp, numElements);
              ELSE
                  UPDATE ME5 SET   ME5.nombreCProd = numElements
                             WHERE ME5.anyo = anyo_temp;
              END IF;
                         -- REINICIEM EL COMPTADOR DE CENTRALS DE PRODUCCIO
              numElements := 0;
          END IF;
        END LOOP;
        CLOSE c_resultat_ME5;  
    
    END IF;
   
   COMMIT;  
  
   RSP := 'OK';
   PR_INSERT_LOG(nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
   
     
EXCEPTION
           
	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR';
		PR_INSERT_LOG(nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
      
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR A PR_ACTUALITZAR_ME5 - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;
	
END;
/