CREATE OR REPLACE
PROCEDURE PR_MODIFICA_CENTRALPROD (

p_idCProd IN CENTRALPROD.idCProd%TYPE,
p_adreca IN CENTRALPROD.adreca%TYPE := '*',
p_codiPostal IN CENTRALPROD.codiPostal%TYPE := '*',
p_localitat IN CENTRALPROD.localitat%TYPE  := '*',
P_pais  IN CENTRALPROD.pais%TYPE  := '*',
p_maxKWh IN CENTRALPROD.maxKWh%TYPE := -1,
p_dataDarreraInspecc IN CENTRALPROD.dataDarreraInspecc%TYPE := '',

RSP OUT NOCOPY TAULALOG.RSP%TYPE
)

/*****************************************************************************
--  PROCEDIMENT:  PR_MODIFICA_CENTRALPROD
--    
--  Modifica les dades d'una central de distribuci segons el seu idCProd.
--  Noms es pot canviar l'adreca, i el mximKWh anual que pot suportat 
--  ja que la resta de dades no sn modificables  a efectes de coherncia 
--  (any fabricaci, idmodel, consums...).
--  
--  En cas que es modifiqui el maxKWh anual, el sistema donar un missatge
--  d'avs si el nou valor s menor a l'anterior ja que pot fer que amb la
--  propera lectura es sobrepassi.
--  
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la 
--  TAULALOG el resultat de la operaci.
--    
--  ENTRADA : P_IDCPROD NUMBER(10), p_ADRECA VARCHAR(30), p_CODIPOSTAL CHAR(5), 
--    p_ LOCALITAT VARCHAR2(30), p_PAIS VARCHAR2(20), p_MAXKWH NUMBER(12,1) ,
--    p_DATADARRERAINSPECC  DATE .
--            
--  SORTIDA : RSP varchar2(300) indicant si ha anat correcte o si ha hagut errors.
--  
--  Si un camp dels parmetres no es vol modificar, en el seu lloc als 
--  parmetres d'entrada s'ha de posar null.
--  
--    RSP = OK
--          
--    RSP = ERROR + motiu, si hi ha algn error.
--  
--    Motius Error :
--         - ERROR: PARAMETRE idCProd NUL.
--         - ERROR: ID CENTRALPROD ' ||p_idCProd||' NO EXISTEIX O JA ESTA DE BAIXA.
--         - ERROR: PARAMETRE MAXKWH REBUT ES NEGATIU.
--         - ERROR: DATA INSPECCIO DONADA: p_dataDarreraInspecc  ES MENOR A LA QUE HI HA : t_dataDarreraInspecc.
--         - Altres
******************************************************************************/



IS
    
    e_idCProd EXCEPTION;
    e_idCProd_nul EXCEPTION;
    e_maxKWh_negatiu EXCEPTION;
    e_dataDarreraInspecc_erronia EXCEPTION;
    
    -- variables insert taulaLOG   
    nomProcediment taulaLog.procExecutat%TYPE := 'PR_MODIFICA_CENTRALPROD';
    paramEntrada taulaLog.paramEntrada%TYPE := 'P_idCProd: ' || p_idCProd || ',p_adreca: '|| p_adreca
              ||',  p_codipostal: '|| p_codipostal||', p_localitat: '||p_localitat
              || ', p_pais '||p_pais|| ', p_maxKWh: '||p_maxKWh ||', p_dataDarreraInspecc: '||p_dataDarreraInspecc;
    paramSortida taulaLog.paramSortida%TYPE := 'RSP';
	
    -- variables d's divers.
    numElements number := 0;
    
    -- variables pels select into de taules.
    t_maxKWh CENTRALPROD.MAXKWH%TYPE;
    t_dataDarreraInspecc CENTRALPROD.dataDarreraInspecc%TYPE;
   
BEGIN
        
    -- VERIFIQUEM QUE EL PARAMETRE idCProd REBUT NO s NULL
    IF (p_idCProd is null) 
      THEN RAISE e_idCProd_nul;
    END IF;
       
    -- VERIFIQUEM QUE EXISTEIX A CENTRALPROD I QUE NO EST DONAT DE BAIXA
    SELECT COUNT(*) INTO numElements 
      FROM CENTRALPROD 
      WHERE p_idCProd = CENTRALPROD.idCProd AND CENTRALPROD.dataBaixa is NULL;
	 	
    IF numElements <>1
      THEN RAISE e_idCProd;
    END IF;
        
    -- VERIFIQUEM QUE LA NOVA DATA DE INSPECCI SIGUI MAJOR A LA EXISTENT
     SELECT CENTRALPROD.DATADARRERAINSPECC INTO t_dataDarreraInspecc
      FROM CENTRALPROD
      WHERE CENTRALPROD.IDCPROD = p_idCprod;
    
    IF t_dataDarreraInspecc > p_dataDarreraInspecc
      THEN RAISE e_dataDarreraInspecc_erronia;
    END IF;
    
    
    -- si parmetre maxkwh s negatiu, llencem error
    IF p_maxKWh < 0 THEN 
      RAISE e_maxKWh_negatiu;
    END IF;
    
    -- si el valor maxkwh que es vol posar s menor al que hi ha
    -- avisem a l'usuari ja que es pot sobrepassar ms fcilment el mxim.
    SELECT CENTRALPROD.MAXKWH INTO t_maxKWh 
          FROM CENTRALPROD
          WHERE CENTRALPROD.idCProd = p_idCProd;
          
    IF t_maxKWh > p_maxKWh
      THEN DBMS_OUTPUT.PUT_LINE('AVIS : MAXKWH NOU ES MENOR AL QUE HI HAVIA');
    END IF;
    
      
    -- SI NO HI HA ERRORS, UPDATEM.
    -- NOMS CANVIEN DE VALOR ELS CAMPS DELS PARMETRES DIFERENTS DE NULL.
    
    UPDATE CENTRALPROD SET adreca = DECODE(p_adreca,'*', adreca , NULL, adreca , p_adreca),
                         codiPostal = DECODE (p_codiPostal, '*', codiPostal, Null , codiPostal, p_codiPostal),
                         localitat = DECODE (p_localitat, '*', localitat, Null , localitat, p_localitat),
                         pais = DECODE (p_pais, '*', pais, Null , pais, p_pais),
                         maxKWh = DECODE (p_maxKWh, -1, maxKWh, Null , maxKWh, p_maxKWh),
                         dataDarreraInspecc = DECODE (p_dataDarreraInspecc, '', dataDarreraInspecc, Null , dataDarreraInspecc, p_dataDarreraInspecc)
                     WHERE CENTRALPROD.idCProd = p_idCProd;
                
                         
	
        COMMIT;
        
        
        RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
        
        DBMS_OUTPUT.PUT_LINE('CENTRALPROD ID '||p_idCProd||' MODIFICADA.');
        
	
EXCEPTION

	WHEN e_idCProd_nul THEN
        	RSP := 'ERROR: PARAMETRE idCProd NUL';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
        WHEN e_idCProd THEN
        	RSP := 'ERROR: ID CENTRALPROD ' ||p_idCProd||' NO EXISTEIX O JA ESTA DE BAIXA';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
       
       WHEN e_dataDarreraInspecc_erronia THEN
        	RSP := 'ERROR: DATA INSPECCIO DONADA: '||p_dataDarreraInspecc||' ES MENOR A LA QUE HI HA : '||t_dataDarreraInspecc;
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
       
       WHEN e_maxKWh_negatiu THEN
        	RSP := 'ERROR: PARAMETRE MAXKWH REBUT ES NEGATIU';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
       
	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR -PR_MODIFICA_CENTRALPROD  '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;
		
END;
/