CREATE OR REPLACE
PROCEDURE PR_MODIFICA_CLIENT (

p_idClient IN CLIENT.idClient%TYPE,
p_adreca IN CLIENT.adreca%TYPE := '*',
p_codiPostal IN CLIENT.codiPostal%TYPE := '*',
p_localitat IN CLIENT.localitat%TYPE  := '*',
P_pais  IN CLIENT.pais%TYPE  := '*',

RSP OUT NOCOPY TAULALOG.RSP%TYPE
)

/*****************************************************************************
-- PROCEDIMENT:  PR_MODIFICA_CLIENT
--    
--  Modifica les dades d'un client. Noms es pot modificar les dades corresponents
--  a l'adrea.
--  
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la 
--  TAULALOG el resultat de la operaci.
--    
--  ENTRADA : P_IDCLIENT NUMBER(10), p_ADRECA VARCHAR(30), p_CODIPOSTAL CHAR(5 , 
--    p_LOCALITAT  VARCHAR2(30) , p_ PAIS  VARCHAR2(20) .
--            
--  SORTIDA : RSP VARCHAR2(300) indicant si ha anat correcte o amb errors.
--  
--  Si un camp dels parmetres no es vol modificar, en el seu lloc als 
--  parmetres d'entrada s'ha de posar null.
--  
--    RSP = OK
--          
--    RSP = ERROR + motiu, si hi ha algn error.
--  
--    Motius Error :
--         - ERROR: PARAMETRE idClient NUL.
--         - ERROR: ID CLIENT p_idClient NO EXISTEIX O ESTA DE BAIXA.
--         - Altres
******************************************************************************/



IS
    
    e_idClient EXCEPTION;
    e_idClient_nul EXCEPTION;
    
       
    -- variables insert taulaLOG   
    nomProcediment taulaLog.procExecutat%TYPE := 'PR_MODIFICA_CLIENT';
    paramEntrada taulaLog.paramEntrada%TYPE := 'P_idClient: ' || p_idClient || ', p_adreca: '|| p_adreca
              ||', p_codipostal: '|| p_codipostal||', p_localitat: '||p_localitat
              || ',p_pais: '||p_pais;
    paramSortida taulaLog.paramSortida%TYPE := 'RSP';
	
    -- variables d's divers.
    numElements number := 0;
   
   
BEGIN
        
    -- VERIFIQUEM QUE EL PARAMETRE idClient REBUT NO s NULL
    IF (p_idClient is null) 
      THEN RAISE e_idClient_nul;
    END IF;
       
    -- VERIFIQUEM QUE EXISTEIX A CLIENT I QUE NO EST DONAT DE BAIXA
    SELECT COUNT(*) INTO numElements 
      FROM CLIENT 
      WHERE p_idClient = CLIENT.idClient AND CLIENT.dataBaixa is NULL;
	 	
    IF numElements <>1
      THEN RAISE e_idClient;
    END IF;
        
   
    -- SI NO HI HA ERRORS, UPDATEM.
    -- NOMS CANVIEN DE VALOR ELS CAMPS DELS PARMETRES DIFERENTS DE NULL.
    
    UPDATE CLIENT SET adreca = DECODE(p_adreca,'*', adreca , NULL, adreca , p_adreca),
                         codiPostal = DECODE (p_codiPostal, '*', codiPostal, Null , codiPostal, p_codiPostal),
                         localitat = DECODE (p_localitat, '*', localitat, Null , localitat, p_localitat),
                         pais = DECODE (p_pais, '*', pais, Null , pais, p_pais)
                         
                     WHERE CLIENT.idClient = p_idClient;
   	
 COMMIT;
        
        RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
        DBMS_OUTPUT.PUT_LINE('CLIENT ID '||p_idClient||' MODIFICAT');
   	
EXCEPTION

	WHEN e_idClient_nul THEN
        	RSP := 'ERROR: PARAMETRE idClient NUL';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
        WHEN e_idClient THEN
        	RSP := 'ERROR: ID CLIENT ' ||p_idClient||' NO EXISTEIX O ESTA DE BAIXA';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
      
	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR -PR_MODIFICA_CLIENT  '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;
		
END;
/