CREATE OR REPLACE
PROCEDURE PR_MODIFICA_COMPTADOR (

p_idComptador IN comptador.idComptador%TYPE,
p_adreca IN comptador.adreca%TYPE := '*',
p_codiPostal IN comptador.codiPostal%TYPE := '*',
p_localitat IN comptador.localitat%TYPE  := '*',
P_pais  IN comptador.pais%TYPE  := '*',

RSP OUT NOCOPY TAULALOG.RSP%TYPE
)

/*****************************************************************************
--  PROCEDIMENT:  PR_MODIFICA_COMPTADOR
--    
--  Modifica les dades d'un comptador donat un idComptador.
--  Noms es pot canviar l'adreca ja que la resta de dades no sn modificables
--  a efectes de coherncia (any fabricaci, idmodel, consums...)
--  
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la 
--  TAULALOG el resultat de la operaci.
--    
--  ENTRADA: P_IDCOMPTADOR NUMBER(10), p_ADRECA VARCHAR(30),p_CODIPOSTAL CHAR(5)
--        , p_ LOCALITAT  VARCHAR2(30), p_PAIS  VARCHAR2(20).
--            
--  SORTIDA : RSP varchar2(300) indicant si ha anat correcte o amb errors.
--  
--  Si un camp dels parmetres no es vol modificar, en el seu lloc als 
--  parmetres d'entrada s'ha de posar null.
--  
--    RSP = OK
--          
--    RSP = ERROR + motiu, si hi ha algn error.
--  
--    Motius Error :
--         - ERROR: PARAMETRE IDCOMPTADOR NUL.
--         - ERROR: ID COMPTADOR p_idComptador NO EXISTEIX O JA ESTA DE BAIXA.
--         - Altres
******************************************************************************/


IS
    
    e_idComptador EXCEPTION;
    e_idComptador_nul EXCEPTION;
       
    -- variables insert taulaLOG   
    nomProcediment taulaLog.procExecutat%TYPE := 'PR_MODIFICA_COMPTADOR';
    paramEntrada taulaLog.paramEntrada%TYPE := 'P_idComptador: ' || p_idComptador || ', p_adreca: '|| p_adreca
              ||',  p_codipostal: '|| p_codipostal||', p_localitat: '||p_localitat|| ', p_pais: '||p_pais;
    paramSortida taulaLog.paramSortida%TYPE := 'RSP';
	
    
    -- variables d's divers.
    numElements number := 0;
    
   
BEGIN
        
    -- VERIFIQUEM QUE EL PARAMETRE IDCOMPTADOR REBUT NO s NULL
    IF (p_idComptador is null) 
      THEN RAISE e_idComptador_nul;
    END IF;
       
    -- VERIFIQUEM QUE EXISTEIX A COMPTADOR I QUE NO EST DONAT DE BAIXA
    SELECT COUNT(*) INTO numElements 
      FROM COMPTADOR 
      WHERE p_idComptador = COMPTADOR.idComptador AND COMPTADOR.dataBaixa is NULL;
	 	
    IF numElements <>1
      THEN RAISE e_idComptador;
    END IF;
        
    
    --SI NO HI HA ERRORS, UPDATEM.
    -- NOMS CANVIEN DE VALOR ELS CAMPS DELS PARMETRES DIFERENTS DE NULL.
    
    UPDATE COMPTADOR SET adreca = DECODE(p_adreca,'*', adreca , NULL, adreca , p_adreca),
                         codiPostal = DECODE (p_codiPostal, '*', codiPostal, Null , codiPostal, p_codiPostal),
                         localitat = DECODE (p_localitat, '*', localitat, Null , localitat, p_localitat),
                         pais = DECODE (p_pais, '*', pais, Null , pais, p_pais)
                     WHERE COMPTADOR.IDCOMPTADOR = p_idcomptador;
  	
    COMMIT;
        
        RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
        
        DBMS_OUTPUT.PUT_LINE('COMPTADOR ID '||p_idComptador||' MODIFICAT');
        
	
EXCEPTION

	WHEN e_idComptador_nul THEN
        	RSP := 'ERROR: PARAMETRE IDCOMPTADOR NUL';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
        WHEN e_idComptador THEN
        	RSP := 'ERROR: ID COMPTADOR ' ||p_idComptador||' NO EXISTEIX O JA ESTA DE BAIXA';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
       
	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR -PR_MODIFICA_COMPTADOR  '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;
		
END;
/