CREATE OR REPLACE
PROCEDURE PR_MODIFICA_CONNLINIA (

p_idCDist CONNLINIA.idCDist%TYPE,
p_idLinia CONNLINIA.idLinia%TYPE,
p_idcProd CONNLINIA.idCProd%TYPE,
p_tipusConn CONNLINIA.tipusConn%TYPE := '*',
p_estat CONNLINIA.estat%TYPE := '-1',

RSP OUT NOCOPY TAULALOG.RSP%TYPE
)

/*****************************************************************************
--  PROCEDIMENT:  PR_MODIFICA_CONNLINIA
--    
--  Modifica les dades d'una connexi entre central de distribuci i
--  central de producci a travs d'una linia.
--  
--  Serveix per canviar d'estat actiu (1) a inactiu (0) o viceversa.
--  
--  Tamb per canviar el tipus de connexi Principal, secundria o Baixa.
--  
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la 
--  TAULALOG el resultat de la operaci.
--    
--  ENTRADA : P_IDCDIST NUMBER(10), p_IDLINIA NUMBER(10), p_IDCPROD NUMBER(10), 
--          p_TIPUSCONN  CHAR(1) , p_ESTAT NUMBER(1).
--            
--  SORTIDA : RSP varchar2(300) indicant si ha anat correcte o amb errors.
--  
--  Si un camp dels parmetres no es vol modificar, en el seu lloc als 
--  parmetres d'entrada s'ha de posar null.
--  
--    RSP = OK
--          
--    RSP = ERROR + motiu, si hi ha algn error.
--  
--    Motius Error :
--         - ERROR: PARAMETRES IDCDIST O IDLINIA O IDCPROD NUL.
--         - ERROR: CONNEXIO NO EXISTEIX.
--         - ERROR: PARAMETRES AMB VALORS ERRONIS.
--         - Altres
******************************************************************************/

IS
    
    e_camp_nul EXCEPTION;
    e_noexisteix EXCEPTION;
    e_valors_erronis EXCEPTION;
    
       
    -- variables insert taulaLOG   
    nomProcediment taulaLog.procExecutat%TYPE := 'PR_MODIFICA_CONNLINIA';
    paramEntrada taulaLog.paramEntrada%TYPE := 'P_idCDist: ' || p_idCDist|| ', p_idLinia : ' 
        || p_idLinia|| ', p_idCProd '|| p_idCProd||', p_tipusConn: '|| p_tipusConn||', p_estat: '||p_estat;
    paramSortida taulaLog.paramSortida%TYPE := 'RSP';
	
    -- variables d's divers.
    numElements number := 0;
   
   
BEGIN
        
    -- VERIFIQUEM QUE EL PARAMETRE idCONNLINIA REBUT NO s NULL
    IF (p_idLinia is null) or (p_idCDist is null) or (p_idCProd is null)
      THEN RAISE e_camp_nul;
    END IF;
       
    -- VERIFIQUEM QUE EXISTEIX  CONNLINIA 
    SELECT COUNT(*) INTO numElements 
      FROM CONNLINIA 
      WHERE  P_idCDist = CONNLINIA.IDCDIST
      AND P_idCProd = CONNLINIA.IDCPROD
      AND P_idLinia = CONNLINIA.IDLINIA       ;
	 	
    IF numElements <>1
      THEN RAISE e_noexisteix;
    END IF;
  
   -- VERIFIQUEM QUE EL TIPUS SIGUI PRINCIPAL, SECUNDARI  o BAIXA I ESTAT 0,1
   IF p_tipusconn IS NOT null AND p_tipusConn <> 'P' AND p_tipusConn <> 'S' AND p_tipusConn <> 'B'
     THEN RAISE e_valors_erronis;
   END IF;
   
   IF   p_estat IS NOT null AND p_estat <> 0 AND p_estat <> 1
    THEN RAISE e_valors_erronis;
   END IF;
   
   -- SI NO HI HA ERRORS, UPDATEM.
   -- NOMS CANVIEN DE VALOR ELS CAMPS DELS PARMETRES DIFERENTS DE NULL.
    UPDATE CONNLINIA SET tipusConn = DECODE(p_tipusConn,'*', tipusConn , NULL, tipusConn , p_tipusConn),
                         estat = DECODE (p_estat, -1, estat , Null , estat  , p_estat )
                         
                     WHERE CONNLINIA.IDLINIA = p_idLinia
                        and CONNLINIA.idCProd = p_idcProd
                        AND CONNLINIA.idCDIST = p_idCDist;
   	
 COMMIT;
        
        RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
        DBMS_OUTPUT.PUT_LINE('CONNEXIO CDISTRIBUCIO-LINIA-CPRODUCCIO ('||p_idCDist||'-'||p_idLinia||'-'||p_idCProd||')  MODIFICADA');
        
    
    -- SI AMB LA MODIFICACI, HEM DEIXAT EL COMPTADOR D'AQUELLA CENTRAL DE
    -- DISTRIBUCI SENSE CONNEXIONS ACTIVES, AVISEM A L'USUARI
    SELECT COUNT(*) INTO numElements 
      FROM CONNLINIA
      WHERE CONNLINIA.IDCDIST = p_idCDist
            AND estat = 1;
            
    IF numElements = 0 THEN
      DBMS_OUTPUT.PUT_LINE('AVIS:  LA CENTRAL DISTRIBUCIO ID '||p_idCDist||' S''HA QUEDAT SENSE CONNEXIONS ACTIVES');
    END IF;
        
   	
EXCEPTION

	WHEN e_camp_nul THEN
        	RSP := 'ERROR: PARAMETRES IDCDIST O IDLINIA O IDCPROD NUL';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
        WHEN e_noexisteix THEN
        	RSP := 'ERROR: CONNEXIO NO EXISTEIX';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
      
	WHEN e_valors_erronis THEN
        	RSP := 'ERROR: PARAMETRES AMB VALORS ERRONIS ';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
        
        WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR -PR_MODIFICA_CONNLINIA  '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;
		
END;
/