CREATE OR REPLACE
PROCEDURE PR_MODIFICA_LINIA (

p_idLinia IN LINIA.idLinia%TYPE,
p_maxKWh IN LINIA.maxKWh%TYPE := -1,

RSP OUT NOCOPY TAULALOG.RSP%TYPE
)


/*****************************************************************************
--  PROCEDIMENT:  PR_MODIFICA_LINIA
--    
--  Modifica les dades d'una central de distribuci segons el seu idLinia.
--  Noms es pot canviar el maxKWh suportat.
--  
--  El sistema donar un missatge d'avs si el nou valor de maxKWh s menor a 
-- l'anterior ja que pot fer que amb la propera lectura es sobrepassi.
--  
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la 
--  TAULALOG el resultat de la operaci.
--    
--  ENTRADA : P_IDLINIA NUMBER(10) , p_MAXKWH  NUMBER(12,1) .
--            
--  SORTIDA : RSP VARCHAR(300) indicant si ha anat correcte o si ha hagut errors.
--  
--  Si un camp dels parmetres no es vol modificar, en el seu lloc als 
--  parmetres d'entrada s'ha de posar null.
--  
-    RSP = OK
--         
--   RSP = ERROR + motiu, si hi ha algn error.
--  
--    Motius Error :
--         - ERROR: PARAMETRE idLinia NUL
--         - ERROR: ID LINIA p_idLinia NO EXISTEIX O JA ESTA DE BAIXA
--         - ERROR: PARAMETRE MAXKWH REBUT ES NEGATIU
--         - Altres
******************************************************************************/

IS
    
    e_idLinia EXCEPTION;
    e_idLinia_nul EXCEPTION;
    e_maxKWh_negatiu EXCEPTION;
       
    -- variables insert taulaLOG   
    nomProcediment taulaLog.procExecutat%TYPE := 'PR_MODIFICA_LINIA';
    paramEntrada taulaLog.paramEntrada%TYPE := 'P_idLinia: ' || p_idLinia || ', p_maxKWh: '||p_maxKWh;
    paramSortida taulaLog.paramSortida%TYPE := 'RSP';
	
    -- variables d's divers.
    numElements number := 0;
    maxKWh_temp LINIA.MAXKWH%TYPE;
   
BEGIN
        
    -- VERIFIQUEM QUE EL PARAMETRE idLinia REBUT NO s NULL
    IF (p_idLinia is null) 
      THEN RAISE e_idLinia_nul;
    END IF;
       
    -- VERIFIQUEM QUE EXISTEIX A LINIA I QUE NO EST DONAT DE BAIXA
    SELECT COUNT(*) INTO numElements 
      FROM LINIA 
      WHERE p_idLinia = LINIA.idLinia AND LINIA.dataBaixa is NULL;
	 	
    IF numElements <>1
      THEN RAISE e_idLinia;
    END IF;
        
    
    -- si parmetre maxkwh s negatiu, llencem error
    IF p_maxKWh < 0 THEN 
      RAISE e_maxKWh_negatiu;
    END IF;
    
    -- si el valor maxkwh que es vol posar s menor al que hi ha
    -- avisem a l'usuari ja que es pot sobrepassar ms fcilment el mxim.
    SELECT LINIA.MAXKWH INTO MAXKWH_TEMP 
          FROM LINIA
          WHERE LINIA.idLinia = p_idLinia;
          
    IF MAXkWH_TEMP > p_maxKWh
      THEN DBMS_OUTPUT.PUT_LINE('AVIS : MAXKWH NOU ES MENOR AL QUE HI HAVIA');
    END IF;
    
      
    -- SI NO HI HA ERRORS, UPDATEM.
    -- (es fa amb decode per si en un futur s'afegeixen atributs a la taula)
     
    UPDATE LINIA SET maxKWh = DECODE (p_maxKWh, -1, maxKWh, Null , maxKWh, p_maxKWh)
                     WHERE LINIA.idLinia = p_idLinia;
    	
    COMMIT;
        RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
        
        DBMS_OUTPUT.PUT_LINE('LINIA ID '||p_idLinia||' MODIFICADA');
        
	
EXCEPTION

	WHEN e_idLinia_nul THEN
        	RSP := 'ERROR: PARAMETRE idLinia NUL';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
        WHEN e_idLinia THEN
        	RSP := 'ERROR: ID LINIA ' ||p_idLinia||' NO EXISTEIX O JA ESTA DE BAIXA';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
       
       WHEN e_maxKWh_negatiu THEN
        	RSP := 'ERROR: PARAMETRE MAXKWH REBUT ES NEGATIU';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
       
	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR -PR_MODIFICA_LINIA  '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;
		
END;
/